import os
import math

import requests
from urllib.parse import urljoin
from urllib.parse import urlsplit

def download_temporary_file(api, temporary_file, out_path):
    """ Download a temporary file from Tator to an off-line location.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        temporary_file = api.get_temporary_file(temporary_file_id)
        out_path = f'/tmp/{temporary_file.name}'
        for progress in tator.util.download_temporary_file(api, temporary_file,
                                                      out_path):
            print(f"Download progress: {progress}%")

    :param api: :class:`tator.TatorApi` object.
    :param temporary_file: :class:`tator.models.TemporaryFile` object.
    :param path-like out_path: Path to where to download.
    :returns: Generator that yields progress (0-100).
    """
    config = api.api_client.configuration
    host = config.host
    token = config.api_key['Authorization']
    prefix = config.api_key_prefix['Authorization']
    url = temporary_file.path

    # Supply token here for eventual media authorization
    headers = {
        'Authorization': f'{prefix} {token}',
        'Content-Type': f'application/json',
        'Accept-Encoding': 'gzip',
    }
    CHUNK_SIZE = 2 * 1024 * 1024
    with requests.get(url, stream=True, headers=headers) as r:
        r.raise_for_status()
        total_size = r.headers['Content-Length']
        total_chunks = math.ceil(int(total_size) / CHUNK_SIZE)
        chunk_count = 0
        last_progress = 0
        yield last_progress
        with open(out_path, 'wb') as f:
            for chunk in r.iter_content(chunk_size=CHUNK_SIZE):
                if chunk:
                    f.write(chunk)
                    this_progress = round((chunk_count / total_chunks) *100,1)
                    if this_progress != last_progress:
                        yield this_progress
                        last_progress = this_progress
        yield 100



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class VersionSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bases': 'list[int]',
        'description': 'str',
        'elemental_id': 'str',
        'name': 'str',
        'show_empty': 'bool'
    }

    attribute_map = {
        'bases': 'bases',
        'description': 'description',
        'elemental_id': 'elemental_id',
        'name': 'name',
        'show_empty': 'show_empty'
    }

    def __init__(self, bases=None, description='', elemental_id=None, name=None, show_empty=True, local_vars_configuration=None):  # noqa: E501
        """VersionSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._bases = None
        self._description = None
        self._elemental_id = None
        self._name = None
        self._show_empty = None
        self.discriminator = None

        if bases is not None:
            self.bases = bases
        if description is not None:
            self.description = description
        if elemental_id is not None:
            self.elemental_id = elemental_id
        self.name = name
        if show_empty is not None:
            self.show_empty = show_empty

    @property
    def bases(self):
        """
        Array of other version IDs that are dependencies of this version.

        :return: The bases of this VersionSpec. 
        :rtype: list[int]
        """
        return self._bases

    @bases.setter
    def bases(self, bases):
        """
        Array of other version IDs that are dependencies of this version.

        :param bases: The bases of this VersionSpec.
        :type: list[int]
        """

        self._bases = bases

    @property
    def description(self):
        """
        Description of the version.

        :return: The description of this VersionSpec. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the version.

        :param description: The description of this VersionSpec.
        :type: str
        """

        self._description = description

    @property
    def elemental_id(self):
        """
        Unique ID of an element

        :return: The elemental_id of this VersionSpec. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        Unique ID of an element

        :param elemental_id: The elemental_id of this VersionSpec.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def name(self):
        """
        Name of the version.

        :return: The name of this VersionSpec. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the version.

        :param name: The name of this VersionSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def show_empty(self):
        """
        Whether to show this version on media for which no annotations exist.

        :return: The show_empty of this VersionSpec. 
        :rtype: bool
        """
        return self._show_empty

    @show_empty.setter
    def show_empty(self, show_empty):
        """
        Whether to show this version on media for which no annotations exist.

        :param show_empty: The show_empty of this VersionSpec.
        :type: bool
        """

        self._show_empty = show_empty

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, VersionSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, VersionSpec):
            return True

        return self.to_dict() != other.to_dict()



from uuid import uuid1
import mimetypes
import os
import math
import tempfile
import logging

import requests
from urllib.parse import urlsplit

from .md5sum import md5sum

logger = logging.getLogger(__name__)

def _hosted_md5(url, _request_timeout):
    """ Downloads only enough of file to compute md5 of first part of file
        to a temporary path, then returns md5.
    """
    CHUNK_SIZE = 2 * 1024 * 1024
    MAX_CHUNKS = 5 # Download a maximum of 10MB.
    MAX_RETRIES = 10
    for attempt in range(MAX_RETRIES):
        try:
            with requests.get(url, stream=True, timeout=_request_timeout) as r:
                r.raise_for_status()
                total_size = int(r.headers['Content-Length'])
                total_chunks = math.ceil(total_size / CHUNK_SIZE)
                chunk_count = 0
                with tempfile.NamedTemporaryFile(mode='wb') as temp:
                    for chunk in r.iter_content(chunk_size=CHUNK_SIZE):
                        if chunk:
                            chunk_count += 1
                            temp.write(chunk)
                        if chunk_count >= MAX_CHUNKS:
                            break
                    # Compute the md5 using file size from header and first 100MB.
                    md5 = md5sum(temp.name, total_size)
            break
        except Exception as ex:
            print(ex)
            logger.info(f"Failed to download {url} on attempt {attempt}...")
            if attempt == MAX_RETRIES - 1:
                raise RuntimeError(f"Reached maximum retries on media download!")
    return md5

def import_media(api, type_id, url, md5=None, section=None, fname=None,
                 upload_gid=None, upload_uid=None,chunk_size=2*1024*1024,
                 attributes=None, media_id=None, size=None, _request_timeout=10, section_id=None):
    """ Imports a hosted media file.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        response = tator.util.import_media(api, type_id, url)
        print(response.message)

    :param api: :class:`tator.TatorApi` object.
    :param type_id: Unique integer identifying a media type.
    :param url: URL of the hosted media file.
    :param md5: [Optional] md5 sum of the media.
    :param section: [Optional] Media section to upload to.
    :param section_id: [Optional] Unique integer identifying a section. If given `section` is ignored.
    :param fname: [Optional] Filename to use for upload.
    :param upload_gid: [Optional] Group ID of the upload.
    :param upload_uid: [Optional] Unique ID of the upload.
    :param chunk_size: [Optional] Chunk size in bytes. Default is 2MB.
    :param attributes: [Optional] Attributes to apply to media object.
    :param media_id: [Optional] Unique ID of existing media object.
    :param size: [Optional] Size of the file in bytes. Required if the
        given URL does not accept HEAD requests.
    :param _request_timeout: [Optional] Timeout setting for API requests in seconds.
    :returns: Generator that yields tuple containing progress (0-100) and a
        response. The response is `None` until the last yield, when the response
        is the response object from :meth:`tator.TatorApi.create_media_list` or
        :meth:`tator.TatorApi.transcode`.
    """
    if md5==None:
        # To compute md5, download the first 10MB to temporary file.
        md5 = _hosted_md5(url, _request_timeout)
    if upload_uid is None:
        upload_uid = str(uuid1())
    if upload_gid is None:
        upload_gid = str(uuid1())
    if fname is None:
        fname = os.path.basename(urlsplit(url).path)
    if section is None and section_id is None:
        section="New Files"

    media_type = api.get_media_type(type_id, _request_timeout=_request_timeout)
    project_id = media_type.project
    section_blob = {"section": section} if section is not None else {"section_id": section_id}
    spec = {
        'type': type_id,
        'uid': upload_uid,
        'gid': upload_gid,
        'url': url,
        'name': fname,
        **section_blob,
        'md5': md5,
        'attributes': attributes,
        'media_id': media_id,
    }
    if size is not None:
        spec['size'] = size
    # Create video or image.
    if media_type.dtype == 'video':
        response = api.transcode(project_id, transcode_spec=spec,
                                 _request_timeout=_request_timeout)
    else:
        response = api.create_media_list(
            project_id, body=[spec], _request_timeout=_request_timeout
        )
    return response



import logging
import os
import shutil
import tempfile
import uuid
from textwrap import dedent

import pytest
import tator
from tator.util._upload_file import _upload_file

logger = logging.getLogger(__name__)

def _create_html_file_str() -> str:
    """ Creates a HTML file used by the unit tests in this file
    """

    return dedent("""\
<html>
<head>
<title>CVisionAI Tator Test</title>
</head>
<body>
<p>Hi! This is a test HTML file for CVisionAI's tator-py bindings</p>
</body>
</html>
        """)

def _missing_upload_file(
        host: str,
        token: str,
        project: int) -> None:
    """ Test missing upload file for SaveGenericFile endpoint

    :param host: Project URL
    :param token: User token used for connecting to the host
    :param project: Unique identifier of test project
    """

    tator_api = tator.get_api(host=host, token=token)
    spec = tator.models.GenericFileSpec(name="test.html", upload_url="not_there")
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException):
        tator_api.save_generic_file(project=project, generic_file_spec=spec)

def _upload_test_html_file(
        host: str,
        token: str,
        project: int,
        html_file: str="test.html"):
    """ Uploads the provided manifest file

    :param host: Project URL
    :param token: User token used for connecting to the host
    :param project: Unique identifier of test project
    :param html_file: Local HTML file to be uploaded and saved
    :returns: Response from the save HTML file endpoint
    """

    # Setup the interface to tator
    tator_api = tator.get_api(host=host, token=token)

    # Create the temporary HTML file
    fd, local_file = tempfile.mkstemp()
    try:
        with os.fdopen(fd, 'w') as file_handle:
            file_handle.write(_create_html_file_str())

        # Upload the file
        logger.info(f"Created temporary HTML file: {local_file}")
        for progress, upload_info in _upload_file(tator_api, project, local_file):
            logger.info(f"Upload progress: {progress}%")
        url = tator_api.get_download_info(project, {'keys': [upload_info.key]})[0].url

        # Save the uploaded file to the project using the save html endpoint
        spec = tator.models.GenericFileSpec(name=html_file, upload_url=url)
        response = tator_api.save_generic_file(project=project, generic_file_spec=spec)

    finally:
        os.remove(local_file)

    return response

def test_save_html_file(
        host: str,
        token: str,
        project: int) -> None:
    """ Unit test for the SaveGenericFile endpoint

    Unit testing of the save generic file endpoint involvest the following:
    - Provide something where the upload file doesn't exist
    - Upload two of the same files (have the same name, but will result in two separate files)

    :param host: Project URL
    :param token: User token used for connecting to the host
    :param project: Unique identifier of test project
    """

    _missing_upload_file(host=host, token=token, project=project)

    response_1 = _upload_test_html_file(
        host=host, token=token, project=project, html_file='test_nodupe.html')
    assert os.path.basename(response_1.url) != "test_nodupe.html"
    url_1 = os.path.basename(response_1.url)
    response_2 = _upload_test_html_file(
        host=host, token=token, project=project, html_file='test_nodupe.html')
    assert os.path.basename(response_2.url) != "test_nodupe.html"
    url_2 = os.path.basename(response_2.url)
    assert url_1 != url_2

def test_dashboard_endpoint(host: str, token: str, project: int) -> None:
    """ Unit tests for the dashboard endpoint

    :param host: Project URL
    :param token: User token used for connecting to the host
    :param project: Unique identifier of test project
    """

    tator_api = tator.get_api(host=host, token=token)

    # Test creating dashboards
    response = _upload_test_html_file(host=host, token=token, project=project)
    spec = tator.models.Applet(
        name="Test Dashboard 1",
        project=project,
        description="",
        html_file=response.url,
        categories=[])

    response = tator_api.register_applet(project=project, applet_spec=spec)
    dashboard = tator_api.get_applet(id=response.id)
    assert dashboard.name == spec.name
    assert dashboard.project == spec.project
    assert dashboard.description == spec.description
    assert dashboard.categories == spec.categories

    response = _upload_test_html_file(host=host, token=token, project=project)
    spec = tator.models.Applet(
        name="Test Dashboard 2",
        project=project,
        description="Test Dashboard 2",
        html_file=response.url,
        categories=["category"])

    response = tator_api.register_applet(project=project, applet_spec=spec)
    dashboard = tator_api.get_applet(id=response.id)
    assert dashboard.name == spec.name
    assert dashboard.project == spec.project
    assert dashboard.description == spec.description
    assert dashboard.categories == spec.categories

    response = _upload_test_html_file(host=host, token=token, project=project)
    spec = tator.models.Applet(
        name="Test Dashboard 3",
        project=project,
        description="Test Dashboard 3",
        html_file=response.url,
        categories=["test1", "test2"])

    response = tator_api.register_applet(project=project, applet_spec=spec)
    dashboard = tator_api.get_applet(id=response.id)
    assert dashboard.name == spec.name
    assert dashboard.project == spec.project
    assert dashboard.description == spec.description
    assert dashboard.categories == spec.categories

    # Test retrieving the dashboard list
    dashboards = tator_api.get_applet_list(project=project)
    assert len(dashboards) == 3

    # Test patching a dashboard's fields
    response = _upload_test_html_file(host=host, token=token, project=project)
    html_url = response.url
    dashboard_id = dashboard.id
    original = tator_api.get_applet(id=dashboard.id)
    update = {
      "name": "Another name",
      "description": "Another description",
      "categories": ["a","b","c","d"],
      "html_file": html_url}
    tator_api.update_applet(id=dashboard_id, applet_spec=update)
    dashboard = tator_api.get_applet(id=dashboard_id)
    assert dashboard.name == update["name"]
    assert dashboard.description == update["description"]
    assert dashboard.categories == update["categories"]
    assert dashboard.html_file != original.html_file

    # Test deleting a dashboard
    tator_api.delete_applet(id=dashboard.id)
    dashboards = tator_api.get_applet_list(project=project)
    assert len(dashboards) == 2



import logging
import tator
import cv2
import tempfile
import pytesseract

import numpy as np

logger = logging.getLogger(__name__)

def test_get_clip(host: str,
                 token: str,
                 count_video: int):
  api = tator.get_api(host, token)
  media = api.get_media(count_video, presigned=3600)
  with tempfile.TemporaryDirectory() as td:
    util = tator.util.MediaUtil(td)
    # This load takes a tator object
    util.load_from_media_object(media, quality=None)

    concat_path,_ = util.get_clip([(25, 49)])
    reader = cv2.VideoCapture(concat_path)
    ok,frame = reader.read()
    expected=25
    while ok:
      img=cv2.cvtColor(frame,cv2.COLOR_BGR2RGB)
      text = pytesseract.image_to_string(img)
      _,val=text.strip().split('=')
      val = val.replace('/','')
      assert(int(val) == expected)
      expected += 1
      ok,frame = reader.read()

    # Check a non-contiguous video
    concat_path,_ = util.get_clip([(25, 49), (100,149)])
    reader = cv2.VideoCapture(concat_path)
    ok,frame = reader.read()
    expected=[*list(np.linspace(25,49,25).astype(np.int32)),
              *list(np.linspace(100,149,50).astype(np.int32))]
    expected_idx = 0
    while ok:
      img=cv2.cvtColor(frame,cv2.COLOR_BGR2RGB)
      text = pytesseract.image_to_string(img)
      _,val=text.strip().split('=')
      val = val.replace('/','')
      assert(int(val) == expected[expected_idx])
      expected_idx += 1
      ok,frame = reader.read()


def test_get_tile_image(host: str,
                        token: str,
                        count_video: int):
  api = tator.get_api(host, token)
  media = api.get_media(count_video, presigned=3600)
  with tempfile.TemporaryDirectory() as td:
    util = tator.util.MediaUtil(td)
    # This load takes a tator object
    util.load_from_media_object(media, quality=None)

    tile=util.get_tile_image([1])
    img = cv2.imread(tile)
    text = pytesseract.image_to_string(img)
    _,val=text.strip().split('=')
    val = val.replace('/','')
    assert(int(val) == 1)

    tile=util.get_tile_image([123,231,100,50])
    img = cv2.imread(tile)
    text = pytesseract.image_to_string(img)
    val=text.strip()
    val = val.replace('/','').split('\n')
    assert(val[0] == "Frame=123 Frame=231")
    assert(val[2] == "Frame=100 Frame=50")



def test_get_animation(host: str,
                       token: str,
                       count_video: int):
  api = tator.get_api(host, token)
  media = api.get_media(count_video, presigned=3600)
  with tempfile.TemporaryDirectory() as td:
    util = tator.util.MediaUtil(td)
    # This load takes a tator object
    util.load_from_media_object(media, quality=None)

    random_frames = [25,39,40,3,100,300,10,44,137] 
    animation_path = util.get_animation(random_frames, 1, render_format='mp4')
    reader = cv2.VideoCapture(animation_path)
    ok,frame = reader.read()
    e_idx = 0
    while ok:
      img=cv2.cvtColor(frame,cv2.COLOR_BGR2RGB)
      text = pytesseract.image_to_string(img)
      _,val=text.strip().split('=')
      val = val.replace('/','')
      assert(int(val) == random_frames[e_idx])
      e_idx += 1
      ok,frame = reader.read()


# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class UploadPart(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'e_tag': 'str',
        'part_number': 'int'
    }

    attribute_map = {
        'e_tag': 'ETag',
        'part_number': 'PartNumber'
    }

    def __init__(self, e_tag=None, part_number=None, local_vars_configuration=None):  # noqa: E501
        """UploadPart - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._e_tag = None
        self._part_number = None
        self.discriminator = None

        if e_tag is not None:
            self.e_tag = e_tag
        if part_number is not None:
            self.part_number = part_number

    @property
    def e_tag(self):
        """
        Content of ETag header returned from PUT request for an upload part. NOTE: key is case-sensitive!

        :return: The e_tag of this UploadPart. 
        :rtype: str
        """
        return self._e_tag

    @e_tag.setter
    def e_tag(self, e_tag):
        """
        Content of ETag header returned from PUT request for an upload part. NOTE: key is case-sensitive!

        :param e_tag: The e_tag of this UploadPart.
        :type: str
        """

        self._e_tag = e_tag

    @property
    def part_number(self):
        """
        Part number for the given ETag response header. NOTE: key is case-sensitive!

        :return: The part_number of this UploadPart. 
        :rtype: int
        """
        return self._part_number

    @part_number.setter
    def part_number(self, part_number):
        """
        Part number for the given ETag response header. NOTE: key is case-sensitive!

        :param part_number: The part_number of this UploadPart.
        :type: int
        """

        self._part_number = part_number

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UploadPart):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UploadPart):
            return True

        return self.to_dict() != other.to_dict()



""" Media manipulation routines

Example:

import tator
api = tator.get_api(token=token)
media = api.get_media(media_id,presigned=3600)
util = tator.util.MediaUtil()

# This load takes a tator object
util.load_from_media_object(media, quality=None)

# path now contains a disk path to an mp4 in the temporary staging area
# containing the frames AT LEAST the frames 25,49
# clip_info can be used to interrogate what frames are present
path, clip_info = util.get_clip([(25,49)])

# Example output:
# ('/tmp/concat.mp4', [{'frame_start': 25, 'num_frames': 25},
#                      {'frame_start': 50, 'num_frames': 25}])

"""
import os
import json
import subprocess
import math
import io
import sys

import requests


def _check_is_presigned(path):
  if path.startswith('http') is False:
    raise Exception("Media object doesn't appear to have presigned URLs present.")
  return path

class MediaUtil:
  """ Top-level class to construct to handle per-media manipulations """
  def __init__(self, temp_dir):
    """ :param temp_dir: Path top use as a working directory for temporary files """
    self._temp_dir = temp_dir
    # If available we only attempt to fetch
    # the part of the file we need to

    # Initialize privates
    self._segment_info = None
    self._fps = None
    self._video_file = None
    self._external_fetch = None
    self._height = None
    self._width = None
    self._start_bias_frame = None
    self._moof_data = None

  def load_from_url(self, video_url, segment_url, height, width, fps):
    """ Setup the MediaUtil instance with primitive types

    :param video_url: Publically accessible URL to the raw segmented mp4 file to use
    :param segment_url: Publically accessible URL for the segmentation map file. 
    :param height: The vertical resolution of the video
    :param width: The horizontal resolution of the video
    :param fps: The frames per second of the video
    """
    self._video_file = video_url
    f_p = io.BytesIO()
    download_request = requests.get(segment_url)
    for chunk in download_request.iter_content(chunk_size=1*1024*1024):
      f_p.write(chunk)
    f_p.seek(0)
    self._segment_info = json.loads(f_p.getvalue().decode('utf-8'))
    self._moof_data = [(i, x) for i, x in enumerate(self._segment_info
                                                        ['segments']) if x['name'] == 'moof']
    self._start_bias_frame = 0
    if self._moof_data[0][1]["frame_start"] > 0:
      self._start_bias_frame = self._moof_data[0][1]["frame_start"]
    self._height = height
    self._width = width
    self._fps = fps

  def load_from_media_object(self, media_obj, quality=None):
    """ Initialize the MediaUtil with a tator media object (loaded with presigned URLs)

      - image or video, no live or multi

      :param media_obj: The a :ref models.Media: object with presigned URLs
      :param quality: Vertical resolution to use as source, will find closest match. 
                      Defaults to highest.
    """
    media = media_obj.to_dict()
    if "streaming" in media['media_files']:
      self._fps = media['fps']
      if quality is None:
        # Select highest quality if not specified
        highest_res = -1
        quality_idx = 0
        for idx, media_info in enumerate(media['media_files']["streaming"]):
          if media_info['resolution'][0] > highest_res:
            highest_res = media_info['resolution'][0]
            quality_idx = idx
      else:
        max_delta = sys.maxsize
        for idx, media_info in enumerate(media['media_files']["streaming"]):
          delta = abs(quality-media_info['resolution'][0])
          if delta < max_delta:
            max_delta = delta
            quality_idx = idx
      if 'hls' in media['media_files']["streaming"][quality_idx]:
        self._external_fetch = 'hls'
        self._video_file = media['media_files']["streaming"][quality_idx]["hls"]
      else:
        video_file = _check_is_presigned(media['media_files']["streaming"][quality_idx]["path"])
        height = media['media_files']["streaming"][quality_idx]["resolution"][0]
        width = media['media_files']["streaming"][quality_idx]["resolution"][1]
        segment_file = _check_is_presigned(media['media_files']["streaming"][quality_idx]["segment_info"])
        self.load_from_url(video_file, segment_file, height, width, media['fps'])
    elif "image" in media['media_files']:
      # Select highest quality image that is non AVIF (no ffmpeg support)
      highest_res = -1
      quality_idx = 0
      # only process non AVIF sources
      images = media['media_files']["image"]
      images = [i for i in images if i['mime'] != 'image/avif']
      for idx, media_info in enumerate(images):
        if media_info['resolution'][0] > highest_res:
          highest_res = media_info['resolution'][0]
          quality_idx = idx
      # Image
      self._video_file = images[quality_idx]["path"]
      self._height = media['height']
      self._width = media['width']
    else:
      raise RuntimeError(f"Media {media.id} does not have streaming or image media!")

  def get_clip(self, frame_ranges):
    """ Given a list of frame ranges generate a temporary mp4

      :param frame_ranges: tuple or list of tuples representing (begin,
                                     end) -- range is inclusive!
    """
    if isinstance(frame_ranges, tuple):
      frame_ranges = [frame_ranges]
    if self._external_fetch == 'hls':
      lookup = {}
      segment_info = [] # There are no segment
      for idx,frange in enumerate(frame_ranges):
        temp_out = os.path.join(self._temp_dir, f"{frange[0]}_{frange[1]}.mp4")
        ffmpeg_args = ["ffmpeg",
                       "-f", "hls",
                       "-y",
                       "-i", self._video_file,
                       "-ss", self._frame_to_time_str(frange[0], None),
                       "-frames:v", str(frange[1]-frange[0]),
                       temp_out]
        proc = subprocess.run(ffmpeg_args, check=True, capture_output=True)
        lookup = {"{frange[0]}_{frange[1]}.mp4": (None, temp_out)}
    else:
      impacted_segments = self._get_impacted_segments_from_ranges(frame_ranges)
      assert not impacted_segments is None, "Unable to calculate impacted video segments"
      lookup, segment_info = self._make_temporary_videos(impacted_segments)

    with open(os.path.join(self._temp_dir, "vid_list.txt"), "w") as vid_list:
      for idx, (_, f_p) in enumerate(lookup.values()):
        mux_0 = os.path.join(self._temp_dir, f"{idx}_0.mp4")
        args = ["ffmpeg",
                "-y",
                "-i", f_p,
                "-c", "copy",
                "-muxpreload", "0",
                "-muxdelay", "0",
                mux_0]
        proc = subprocess.run(args, check=True, capture_output=True)
        vid_list.write(f"file '{mux_0}'\n")

    output_file = os.path.join(self._temp_dir, "concat.mp4")
    args = ["ffmpeg",
            "-y",
            "-f", "concat",
            "-safe", "0",
            "-i", os.path.join(self._temp_dir, "vid_list.txt"),
            "-c", "copy",
            output_file]
    proc = subprocess.run(args, check=True, capture_output=True)
    return output_file, segment_info

  def get_tile_image(self, frames, rois=None, tile_size=None,
                     render_format="jpg", force_scale=None):
    """ Generate a tile jpeg of the given frame/rois """
    # Compute tile size if not supplied explicitly
    try:
      if tile_size is not None:
        # check supplied tile size makes sense
        comps = tile_size.split('x')
        if len(comps) != 2:
          raise Exception("Bad Tile Size")
        if int(comps[0])*int(comps[1]) < len(frames):
          raise Exception("Bad Tile Size")
    except Exception as e:
      tile_size = None
      # compute the required tile size
    if tile_size is None:
      width = math.ceil(math.sqrt(len(frames)))
      height = math.ceil(len(frames) / width)
      tile_size = f"{width}x{height}"

    if not self._generate_frame_images(frames, 
                                       rois,
                                       render_format=render_format,
                                       force_scale=force_scale):
      return None

    output_file = None
    if len(frames) > 1:
      # Make a tiled jpeg
      tile_args = ["ffmpeg",
                   "-y",
                   "-i", os.path.join(self._temp_dir, f"%d.{render_format}"),
                   "-vf", f"tile={tile_size}",
                   "-q:v", "3",
                   os.path.join(self._temp_dir, f"tile.{render_format}")]
      proc = subprocess.run(tile_args, check=True, capture_output=True)
      if proc.returncode == 0:
        output_file = os.path.join(self._temp_dir, f"tile.{render_format}")
    else:
      output_file = os.path.join(self._temp_dir, f"0.{render_format}")

    return output_file

  def get_animation(self, frames, fps, roi=None, render_format='mp4', force_scale=None):
    """ Return an animation of frames at a given FPS.

        :param  frames: list of frames
        :param fps: FPS of output animation """
    if not self._generate_frame_images(frames, roi,
                                       render_format="jpg",
                                       force_scale=force_scale):
      return None

    mp4_args = ["ffmpeg",
                "-y",
                "-framerate", str(fps),
                "-i", os.path.join(self._temp_dir, "%d.jpg"),
                os.path.join(self._temp_dir, "temp.mp4")]
    proc = subprocess.run(mp4_args, check=True, capture_output=True)

    if render_format == 'mp4':
      return os.path.join(self._temp_dir, "temp.mp4")

    # Convert temporary mp4 into a gif
    gif_args = ["ffmpeg",
                "-y",
                "-i", os.path.join(self._temp_dir, "temp.mp4"),
                "-filter_complex", "[0:v] split [a][b];[a] palettegen"
                " [p];[b][p] paletteuse",
                os.path.join(self._temp_dir, "animation.gif")]
    proc = subprocess.run(gif_args, check=True, capture_output=True)
    return os.path.join(self._temp_dir, "animation.gif")

  def _get_impacted_segments(self, frames):
    """ (Internal function) Return the segment info for the requested frame

    :param frames: List of frames
    """
    if self._segment_info is None and self._external_fetch != None:
      return None

    segment_list = []
    for frame_str in frames:
      frame_seg = set()
      frame_seg.add(0)
      frame_seg.add(1)
      # We already load the header so ignore those segments
      frame = int(frame_str)
      min_idx = 0
      max_idx = len(self._moof_data)-1
      # Handle frames and files with frame biases
      if frame < self._moof_data[0][1]['frame_start']:
        # Force add the first two segments and all the data in between
        frame_seg.add(self._moof_data[0][0])
        for data_idx in range(self._moof_data[0][0] + 1, self._moof_data[1][0], 1):
          frame_seg.add(data_idx)
        frame_seg.add(self._moof_data[1][0])
        frame_seg = list(frame_seg)
        frame_seg.sort()
        segment_list.append((frame, frame_seg))
        continue

      last_segment = self._moof_data[max_idx][1] #Frame start/stop is in the moof
      if frame >= last_segment['frame_start'] + last_segment['frame_samples']:
        continue

      # Do a binary search for the segment in question
      while min_idx <= max_idx:
        guess_idx = math.floor((max_idx + min_idx) / 2)
        moof = self._moof_data[guess_idx][1]
        if frame < moof['frame_start']:
          max_idx = max_idx - 1
        elif frame >= moof['frame_start'] + moof['frame_samples']:
          min_idx = min_idx + 1
        else:
          frame_seg.add(self._moof_data[guess_idx][0])
          frame_seg.add(self._moof_data[guess_idx][0]+1)
          if frame - moof['frame_start'] > moof['frame_samples'] - 5:
            # Handle boundary conditions
            if guess_idx + 1 in self._moof_data:
              frame_seg.add(self._moof_data[guess_idx+1][0])
              frame_seg.add(self._moof_data[guess_idx+1][0]+1)
          break

      frame_seg = list(frame_seg)
      frame_seg.sort()
      segment_list.append((frame, frame_seg))
    return segment_list

  def _get_impacted_segments_from_ranges(self, frame_ranges):
    """ (Internal function) Return the segment info for the requested frame ranges

    :param frames: List of frame range tuples
    """
    segment_list = []
    for frame_range in frame_ranges:
      begin = frame_range[0]
      end = frame_range[1]
      begin_segments = self._get_impacted_segments([begin])
      end_segments = self._get_impacted_segments([end])
      range_segment_set = set()
      for _, frame_seg in [*begin_segments, *end_segments]:
        for segment in frame_seg:
          range_segment_set.add(segment)
      start_missing = begin_segments[0][1][-1]
      end_missing = end_segments[0][1][2]
      for frame_seg in range(start_missing, end_missing):
        range_segment_set.add(frame_seg)
      range_segment = list(range_segment_set)
      range_segment.sort()
      segment_list.append((frame_range, range_segment))
    return segment_list

  def _make_temporary_videos(self, segment_list):
    """ Return a temporary mp4 for each impacted segment to limit IO to
      cloud storage """
    lookup = {}
    segment_info = []
    for frame, segments in segment_list:
      temp_video = os.path.join(self._temp_dir, f"{frame}.mp4")
      sc_graph = [(0, 0)]
      segment_frame_start = sys.maxsize
      # create a scatter/gather
      for segment_idx in segments:
        segment = self._segment_info['segments'][segment_idx]
        last_io = sc_graph[len(sc_graph)-1]
        if segment.get('frame_start', sys.maxsize) < segment_frame_start:
          segment_frame_start = segment['frame_start']

        if 'frame_samples' in segment:
          segment_info.append({
              'frame_start': segment['frame_start'],
              'num_frames': segment['frame_samples']})

        if last_io[0] + last_io[1] == segment['offset']:
          # merge contigous blocks
          sc_graph[len(sc_graph)-1] = (last_io[0], last_io[1] + segment['size'])
        else:
          # A new block
          sc_graph.append((segment['offset'], segment['size']))

      if segment_frame_start == sys.maxsize:
        segment_frame_start = frame
      lookup[frame] = (segment_frame_start, temp_video)

      with open(temp_video, "wb") as out_fp:
        for scatter in sc_graph:
          start = scatter[0]
          stop = scatter[0] + scatter[1] - 1 # Byte range is inclusive
          download_request = requests.get(self._video_file, 
                                          headers={'range': f'bytes={start}-{stop}'})
          for chunk in download_request.iter_content(chunk_size=1*1024*1024):
            out_fp.write(chunk)

    return lookup, segment_info

  def _frame_to_time_str(self, frame, relative_to=None):
    """ Convert a frame number to a time reference """
    if relative_to:
      frame -= relative_to

    if frame < 0:
      frame += self._start_bias_frame

    total_seconds = frame / self._fps
    hours = math.floor(total_seconds / 3600)
    minutes = math.floor((total_seconds % 3600) / 60)
    seconds = total_seconds % 60
    return f"{hours}:{minutes}:{seconds}"

  def _generate_frame_images(self, frames, rois=None, render_format="jpg", force_scale=None):
    """ Generate a jpg for each requested frame and store in the working directory """
    BATCH_SIZE = 30
    frame_idx = 0
    procs = []
    for idx in range(0, len(frames), BATCH_SIZE):
      batch = [int(frame) for frame in frames[idx:idx+BATCH_SIZE]]
      crop_filter = None
      if rois:
        crop_filter = []
        for c in rois: #pylint: disable=invalid-name
          w = max(0,min(round(c[0]*self._width),self._width)) #pylint: disable=invalid-name
          h = max(0,min(round(c[1]*self._height),self._height)) #pylint: disable=invalid-name
          x = max(0,min(round(c[2]*self._width),self._width)) #pylint: disable=invalid-name
          y = max(0,min(round(c[3]*self._height),self._height)) #pylint: disable=invalid-name
          crop_filter.append(f"crop={w}:{h}:{x}:{y}")
      scale_filter = None
      if force_scale:
        scale_w = force_scale[0]
        scale_h = force_scale[1]
        scale_filter = f"scale={scale_w}:{scale_h}"

      args = ["ffmpeg", "-y"]
      inputs = []
      outputs = []

      # attempt to make a temporary file in a fast manner to speed up AWS access
      impacted_segments = self._get_impacted_segments(batch)
      lookup = {}
      if impacted_segments:
        lookup, _ = self._make_temporary_videos(impacted_segments)

      for batch_idx, frame in enumerate(batch):
        outputs.extend(["-map", f"{batch_idx}:v","-frames:v", "1", "-q:v", "3"])
        video_filters = []
        if crop_filter:
          video_filters.append(crop_filter[frame_idx])
        if scale_filter:
          video_filters.append(scale_filter)
        if video_filters:
          outputs.extend(["-vf", ",".join(video_filters)])

        outputs.append(os.path.join(self._temp_dir,f"{frame_idx}.{render_format}"))
        if frame in lookup:
          inputs.extend(["-ss", self._frame_to_time_str(frame, lookup[frame][0]),
                         "-i", lookup[frame][1]])
        elif self._external_fetch == 'hls':
          inputs.extend(["-ss", self._frame_to_time_str(frame, None),
                         "-f", "hls",
                         "-i", self._video_file])
        else:
          raise ValueError("Failed to find frame {frame} in segmented mp4!")
        frame_idx += 1

      # Now add all the cmds in
      args.extend(inputs)
      args.extend(outputs)
      procs.append(subprocess.run(args, check=True, capture_output=True))
    return any([proc.returncode == 0 for proc in procs])



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class RowProtectionSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'algorithm': 'int',
        'bucket': 'int',
        'file': 'int',
        'group': 'int',
        'hosted_template': 'int',
        'job_cluster': 'int',
        'localization': 'int',
        'media': 'int',
        'organization': 'int',
        'permission': 'int',
        'project': 'int',
        'section': 'int',
        'state': 'int',
        'target_group': 'int',
        'target_organization': 'int',
        'user': 'int',
        'version': 'int'
    }

    attribute_map = {
        'algorithm': 'algorithm',
        'bucket': 'bucket',
        'file': 'file',
        'group': 'group',
        'hosted_template': 'hosted_template',
        'job_cluster': 'job_cluster',
        'localization': 'localization',
        'media': 'media',
        'organization': 'organization',
        'permission': 'permission',
        'project': 'project',
        'section': 'section',
        'state': 'state',
        'target_group': 'target_group',
        'target_organization': 'target_organization',
        'user': 'user',
        'version': 'version'
    }

    def __init__(self, algorithm=None, bucket=None, file=None, group=None, hosted_template=None, job_cluster=None, localization=None, media=None, organization=None, permission=None, project=None, section=None, state=None, target_group=None, target_organization=None, user=None, version=None, local_vars_configuration=None):  # noqa: E501
        """RowProtectionSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._algorithm = None
        self._bucket = None
        self._file = None
        self._group = None
        self._hosted_template = None
        self._job_cluster = None
        self._localization = None
        self._media = None
        self._organization = None
        self._permission = None
        self._project = None
        self._section = None
        self._state = None
        self._target_group = None
        self._target_organization = None
        self._user = None
        self._version = None
        self.discriminator = None

        if algorithm is not None:
            self.algorithm = algorithm
        if bucket is not None:
            self.bucket = bucket
        if file is not None:
            self.file = file
        if group is not None:
            self.group = group
        if hosted_template is not None:
            self.hosted_template = hosted_template
        if job_cluster is not None:
            self.job_cluster = job_cluster
        if localization is not None:
            self.localization = localization
        if media is not None:
            self.media = media
        if organization is not None:
            self.organization = organization
        self.permission = permission
        if project is not None:
            self.project = project
        if section is not None:
            self.section = section
        if state is not None:
            self.state = state
        if target_group is not None:
            self.target_group = target_group
        if target_organization is not None:
            self.target_organization = target_organization
        if user is not None:
            self.user = user
        if version is not None:
            self.version = version

    @property
    def algorithm(self):
        """
        Reference to protected object.

        :return: The algorithm of this RowProtectionSpec. 
        :rtype: int
        """
        return self._algorithm

    @algorithm.setter
    def algorithm(self, algorithm):
        """
        Reference to protected object.

        :param algorithm: The algorithm of this RowProtectionSpec.
        :type: int
        """

        self._algorithm = algorithm

    @property
    def bucket(self):
        """
        Reference to protected object.

        :return: The bucket of this RowProtectionSpec. 
        :rtype: int
        """
        return self._bucket

    @bucket.setter
    def bucket(self, bucket):
        """
        Reference to protected object.

        :param bucket: The bucket of this RowProtectionSpec.
        :type: int
        """

        self._bucket = bucket

    @property
    def file(self):
        """
        Reference to protected object.

        :return: The file of this RowProtectionSpec. 
        :rtype: int
        """
        return self._file

    @file.setter
    def file(self, file):
        """
        Reference to protected object.

        :param file: The file of this RowProtectionSpec.
        :type: int
        """

        self._file = file

    @property
    def group(self):
        """
        Group this rule applies to.

        :return: The group of this RowProtectionSpec. 
        :rtype: int
        """
        return self._group

    @group.setter
    def group(self, group):
        """
        Group this rule applies to.

        :param group: The group of this RowProtectionSpec.
        :type: int
        """

        self._group = group

    @property
    def hosted_template(self):
        """
        Reference to protected object.

        :return: The hosted_template of this RowProtectionSpec. 
        :rtype: int
        """
        return self._hosted_template

    @hosted_template.setter
    def hosted_template(self, hosted_template):
        """
        Reference to protected object.

        :param hosted_template: The hosted_template of this RowProtectionSpec.
        :type: int
        """

        self._hosted_template = hosted_template

    @property
    def job_cluster(self):
        """
        Reference to protected object.

        :return: The job_cluster of this RowProtectionSpec. 
        :rtype: int
        """
        return self._job_cluster

    @job_cluster.setter
    def job_cluster(self, job_cluster):
        """
        Reference to protected object.

        :param job_cluster: The job_cluster of this RowProtectionSpec.
        :type: int
        """

        self._job_cluster = job_cluster

    @property
    def localization(self):
        """
        Reference to protected object.

        :return: The localization of this RowProtectionSpec. 
        :rtype: int
        """
        return self._localization

    @localization.setter
    def localization(self, localization):
        """
        Reference to protected object.

        :param localization: The localization of this RowProtectionSpec.
        :type: int
        """

        self._localization = localization

    @property
    def media(self):
        """
        Reference to protected object.

        :return: The media of this RowProtectionSpec. 
        :rtype: int
        """
        return self._media

    @media.setter
    def media(self, media):
        """
        Reference to protected object.

        :param media: The media of this RowProtectionSpec.
        :type: int
        """

        self._media = media

    @property
    def organization(self):
        """
        Organization this rule applies to.

        :return: The organization of this RowProtectionSpec. 
        :rtype: int
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """
        Organization this rule applies to.

        :param organization: The organization of this RowProtectionSpec.
        :type: int
        """

        self._organization = organization

    @property
    def permission(self):
        """
        Bitfield represented by PermissionMask

        :return: The permission of this RowProtectionSpec. 
        :rtype: int
        """
        return self._permission

    @permission.setter
    def permission(self, permission):
        """
        Bitfield represented by PermissionMask

        :param permission: The permission of this RowProtectionSpec.
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and permission is None:  # noqa: E501
            raise ValueError("Invalid value for `permission`, must not be `None`")  # noqa: E501

        self._permission = permission

    @property
    def project(self):
        """
        Reference to protected object.

        :return: The project of this RowProtectionSpec. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Reference to protected object.

        :param project: The project of this RowProtectionSpec.
        :type: int
        """

        self._project = project

    @property
    def section(self):
        """
        Reference to protected object.

        :return: The section of this RowProtectionSpec. 
        :rtype: int
        """
        return self._section

    @section.setter
    def section(self, section):
        """
        Reference to protected object.

        :param section: The section of this RowProtectionSpec.
        :type: int
        """

        self._section = section

    @property
    def state(self):
        """
        Reference to protected object.

        :return: The state of this RowProtectionSpec. 
        :rtype: int
        """
        return self._state

    @state.setter
    def state(self, state):
        """
        Reference to protected object.

        :param state: The state of this RowProtectionSpec.
        :type: int
        """

        self._state = state

    @property
    def target_group(self):
        """
        Reference to protected object.

        :return: The target_group of this RowProtectionSpec. 
        :rtype: int
        """
        return self._target_group

    @target_group.setter
    def target_group(self, target_group):
        """
        Reference to protected object.

        :param target_group: The target_group of this RowProtectionSpec.
        :type: int
        """

        self._target_group = target_group

    @property
    def target_organization(self):
        """
        Reference to protected object.

        :return: The target_organization of this RowProtectionSpec. 
        :rtype: int
        """
        return self._target_organization

    @target_organization.setter
    def target_organization(self, target_organization):
        """
        Reference to protected object.

        :param target_organization: The target_organization of this RowProtectionSpec.
        :type: int
        """

        self._target_organization = target_organization

    @property
    def user(self):
        """
        User this rule applies to.

        :return: The user of this RowProtectionSpec. 
        :rtype: int
        """
        return self._user

    @user.setter
    def user(self, user):
        """
        User this rule applies to.

        :param user: The user of this RowProtectionSpec.
        :type: int
        """

        self._user = user

    @property
    def version(self):
        """
        Reference to protected object.

        :return: The version of this RowProtectionSpec. 
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        Reference to protected object.

        :param version: The version of this RowProtectionSpec.
        :type: int
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RowProtectionSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RowProtectionSpec):
            return True

        return self.to_dict() != other.to_dict()



from datetime import datetime
import os
from random import randint
import tempfile
from time import sleep
from urllib.parse import parse_qs, urlparse
from uuid import uuid1
import pytest

import tator

def test_get_file(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    with tempfile.TemporaryDirectory() as temp_dir:
        outpath = os.path.join(temp_dir, "video.mp4")
        for progress in tator.download_media(tator_api, video_obj, outpath):
            print(f"Video download progress: {progress}%")
        assert(os.path.exists(outpath))

def test_get_audio(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    audio = video_obj.media_files.audio
    assert len(audio) > 0
    assert audio[0].codec == 'aac'

def test_get_by_id(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)
    other_obj = tator_api.get_media_list_by_id(project, {'ids': [video]})[0]
    assert video_obj.id == other_obj.id
    count = tator_api.get_media_count_by_id(project, {'ids': [video]})
    assert(count == 1)

def test_archive(host, token, project, video):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)

    # Test default value of `archived` is "live"
    assert video_obj.archive_state == "live"

    # Test default `get_media_list` filters on `archive_lifecycle == "all"`
    response = tator_api.get_media_list(project, media_id=[video])
    assert len(response) == 1
    assert response[0].archive_state == "live"

    # Test `get_media_list` with `archive_lifecycle="archived"` doesn't return "live" objects
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="archived")
    assert len(response) == 0

    # Test returning subset of media that is live
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="live")
    assert len(response) == 1
    assert response[0].archive_state == "live"

    # Test returning subset of media that has any `archive_lifecycle` state
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="all")
    assert len(response) == 1
    assert response[0].archive_state == "live"

    # Mark the video to archive
    tator_api.update_media_list(project, {"archive_state": "to_archive", "ids": [video]})

    # Wait for update to propagate to ES
    sleep(2)

    # Test default `get_media_list` filters on `archive_lifecycle == "all"`
    response = tator_api.get_media_list(project, media_id=[video])
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Test `get_media_list` with `archive_lifecycle="archived"` returns archived objects
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="archived")
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Test returning subset of media that is live
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="live")
    assert len(response) == 0

    # Test returning subset of media that has any `archive_state` state
    response = tator_api.get_media_list(project, media_id=[video], archive_lifecycle="all")
    assert len(response) == 1
    assert response[0].archive_state == "to_archive"

    # Additional attempts to set the state to `to_archive` will not change anything
    tator_api.update_media(video, {"archive_state": "to_archive"})
    video_obj = tator_api.get_media(video)
    assert video_obj.archive_state == "to_archive"

    # Setting the `archive_state` to `to_live` when a media is in the state `to_archive` should
    # result in the media's state changing to `live`
    tator_api.update_media(video, {"archive_state": "to_live"})
    video_obj = tator_api.get_media(video)
    assert video_obj.archive_state == "live"

def test_section(host, token, project, video):
    tator_api = tator.get_api(host, token)

    # Create test section
    section_spec = {"name": "Test Section", "tator_user_sections": str(uuid1())}
    response = tator_api.create_section(project, section_spec=section_spec)
    section_id = response.id

    # Update media `primary_section`
    update_spec = {"primary_section": section_id}
    response = tator_api.update_media(
        video, update_spec
    )
    media = tator_api.get_media(video)
    assert media.primary_section == section_id
    assert media.attributes["tator_user_sections"] == section_spec["tator_user_sections"]

    # Unset media `primary_section` attribute
    response = tator_api.update_media(
        video, {"primary_section": -1}
    )
    media = tator_api.get_media(video)
    assert media.primary_section == None
    assert media.attributes["tator_user_sections"] == ""


    # Update media `primary_section` attribute with a bulk update
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException):
        response = tator_api.update_media_list(project, media_id=[video], media_bulk_update=update_spec, count=2)
    response = tator_api.update_media_list(project, media_id=[video], media_bulk_update=update_spec, count=1)
    media = tator_api.get_media(video)
    assert media.primary_section == section_id
    assert media.attributes["tator_user_sections"] == section_spec["tator_user_sections"]

    # Unset media `primary_section` attribute with a bulk update
    response = tator_api.update_media_list(
        project,
        media_id=[video],
        media_bulk_update={"primary_section": -1},
    )
    media = tator_api.get_media(video)
    assert media.primary_section == None
    assert media.attributes["tator_user_sections"] == ""

    # Update media `tator_user_sections` attribute
    update_spec = {"attributes": {"tator_user_sections": section_spec["tator_user_sections"]}}
    response = tator_api.update_media(
        video, update_spec
    )
    media = tator_api.get_media(video)
    assert media.primary_section == section_id
    assert media.attributes["tator_user_sections"] == section_spec["tator_user_sections"]

    # Unset media `tator_user_sections` attribute
    response = tator_api.update_media(
        video, {"attributes": {"tator_user_sections": ""}}
    )
    media = tator_api.get_media(video)
    assert media.primary_section == None
    assert media.attributes["tator_user_sections"] == ""

    # Update media `tator_user_sections` attribute with a bulk update
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException):
        response = tator_api.update_media_list(project, media_id=[video], media_bulk_update=update_spec, count=2)
    response = tator_api.update_media_list(project, media_id=[video], media_bulk_update=update_spec, count=1)
    media = tator_api.get_media(video)
    assert media.primary_section == section_id
    assert media.attributes["tator_user_sections"] == section_spec["tator_user_sections"]

    # Unset media `primary_section` attribute with a bulk update
    response = tator_api.update_media_list(
        project,
        media_id=[video],
        media_bulk_update={"attributes": {"tator_user_sections": ""}},
    )
    media = tator_api.get_media(video)
    assert media.primary_section == None
    assert media.attributes["tator_user_sections"] == ""


def test_import_multiple_images(host, token, project, image_type):
    tator_api = tator.get_api(host, token)
    image_url = "https://www.gstatic.com/webp/gallery/1.jpg"
    n_images = 5
    uuid = str(uuid1())

    project_media_count = tator_api.get_media_count(project, type=image_type)

    media_specs = [
        {
            "type": image_type,
            "section": "Multiple image upload",
            "name": f"{uuid}_{idx}.jpg",
            "url": image_url,
            "md5": "",
        }
        for idx in range(n_images)
    ]

    start = datetime.now()
    response = tator_api.create_media_list(project, body=media_specs)
    created_ids = response.id
    duration = (datetime.now() - start).total_seconds()
    assert duration < 5

    assert str(len(media_specs)) in response.message
    assert len(media_specs) == len(created_ids)

    new_project_media_count = -1
    desired_project_media_count = project_media_count + len(media_specs)
    for _ in range(30):
        sleep(1)
        new_project_media_count = tator_api.get_media_count(project, type=image_type)
        if new_project_media_count == desired_project_media_count:
            break
    assert new_project_media_count == desired_project_media_count
    for _ in range(30):
        sleep(1)
        media_list = tator_api.get_media_list(project, type=image_type, media_id=created_ids)

        n_with_media_files = 0
        for media in media_list:
            if media.name.startswith(uuid) and media.media_files is not None:
                n_with_media_files += 1
        if n_with_media_files == n_images:
            break
    assert n_with_media_files == n_images


def parse_url(url):
    parsed_url = urlparse(url)
    return parsed_url.path, parse_qs(parsed_url.query)


def parse_media_files(media):
    return dict(
        parse_url(spec["path"])
        for file_type, file_specs in media.media_files.to_dict().items()
        if file_specs is not None
        for spec in file_specs
        if "path" in spec
    )


def test_presigned_no_cache(host, token, project, video_type, video_file):
    expires_key = "X-Amz-Expires"

    # Set up new video to ensure a clean cache
    tator_api = tator.get_api(host, token)
    uuid_val = str(uuid1())
    attributes = {"test_string": uuid_val}
    for progress, response in tator.util.upload_media(
            tator_api, video_type, video_file, attributes=attributes
    ):
        print(f"Upload video progress: {progress}%")
    print(response.message)

    while True:
        response = tator_api.get_media_list(
            project,
            name='AudioVideoSyncTest_BallastMedia.mp4',
            attribute=[f"test_string::{uuid_val}"],
        )
        print("Waiting for transcode...")
        sleep(2.5)
        if len(response) == 0:
            continue
        if response[0].media_files is None:
            continue
        streaming = response[0].media_files.streaming
        have_archival = response[0].media_files.archival is not None
        if streaming and have_archival and len(streaming) == 4:
            video_id = response[0].id
            break

    # Get initial presigned url
    original_presigned_duration = new_presigned_duration = randint(8640, 86400)
    while new_presigned_duration == original_presigned_duration:
        new_presigned_duration = randint(8640, 86400)

    video_obj = tator_api.get_media(video_id, presigned=original_presigned_duration)

    init_presigned_url = parse_media_files(video_obj)

    # Request a new duration without the `no_cache` flag and assert it returns the same urls
    video_obj = tator_api.get_media(video_id, presigned=new_presigned_duration, no_cache=False)
    no_cache_false_presigned_url = parse_media_files(video_obj)

    for path, query_params in no_cache_false_presigned_url.items():
        assert path in init_presigned_url
        assert int(query_params[expires_key][0]) == original_presigned_duration

    # Request a new duration with the `no_cache` flag and assert it returns new urls
    video_obj = tator_api.get_media(video_id, presigned=new_presigned_duration, no_cache=True)
    no_cache_false_presigned_url = parse_media_files(video_obj)

    for path, query_params in no_cache_false_presigned_url.items():
        assert path in init_presigned_url
        assert int(query_params[expires_key][0]) == new_presigned_duration

    # Request a new duration without the `no_cache` flag again and assert it returns the original
    # cached urls and not the ones from the new duration
    video_obj = tator_api.get_media(video_id, presigned=original_presigned_duration, no_cache=False)
    no_cache_false_presigned_url = parse_media_files(video_obj)

    for path, query_params in no_cache_false_presigned_url.items():
        assert path in init_presigned_url
        assert int(query_params[expires_key][0]) == original_presigned_duration

def test_upload_to_section_id(host, token, project, image_type, image_file):
    tator_api = tator.get_api(host, token)
    section_spec = {
        "name": "Section ID test",
        "tator_user_sections": str(uuid1()),
    }
    section_id = tator_api.create_section(project, section_spec).id
    attributes = {"test_string": str(uuid1())}
    for progress, response in tator.util.upload_media(
            tator_api, image_type, image_file, section_id=section_id, attributes=attributes
    ):
        print(f"Upload image to section by ID progress: {progress}%")
    print(response.message)
    media_list = tator_api.get_media_list(
        project,
        section=section_id,
    )
    assert(len(media_list) == 1)



import json

import tator

def test_clone_membership_same_host(host, token, project, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator_api.whoami()
    user_id = response.id
    memberships = tator_api.get_membership_list(project)
    for membership in memberships:
        if membership.user == user_id:
            break
    try:
        response = tator.util.clone_membership(tator_api, membership.id, clone_project)
    except tator.openapi.tator_openapi.exceptions.ApiException as exc:
        assert(json.loads(exc.body)['message'].startswith('Membership already exists'))

def test_clone_membership_different_host(host, token, project, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator_api.whoami()
    user_id = response.id
    memberships = tator_api.get_membership_list(project)
    for membership in memberships:
        if membership.user == user_id:
            break
    try:
        response = tator.util.clone_membership(tator_api, membership.id, clone_project, tator_api)
    except tator.openapi.tator_openapi.exceptions.ApiException as exc:
        assert(json.loads(exc.body)['message'].startswith('Membership already exists'))




def clone_membership(src_api, src_membership_id, dest_project, user_mapping=None, dest_api=None):
    """ Clone membership.

    This can be used to clone memberships from one project to another or from one
    host to another.

    Example for different host:

    .. code-block:: python

        src_api = tator.get_api(host, token)
        dest_api = tator.get_api(other_host, other_token)
        src_membership_id = 1
        dest_project = 1
        user_mapping = {1: 10} # Mapping between user ID on source host and destination host.
        response = tator.util.clone_membership(src_api, src_membership_id, dest_project,
                                               user_mapping, dest_api)
        print(response.message)

    Example for same host:

    .. code-block:: python

        api = tator.get_api(host, token)
        src_membership_id = 1
        dest_project = 1
        response = tator.util.clone_membership(src_api, src_membership_id, dest_project)
        print(response.message)

    :param src_api: :class:`tator.TatorApi` object corresponding to source host or only
        host if this is a clone on same host.
    :param src_membership_id: Unique integer identifying membership to clone.
    :param dest_project: Unique integer identifying destination project.
    :param user_mapping: If cloning to different host, the mapping between source and destination
        user IDs.
    :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
    :returns: Response from membership creation request.
    """
    membership_obj = src_api.get_membership(src_membership_id)
    spec = {'permission': membership_obj.permission}
    if dest_api is None:
        spec['user'] = membership_obj.user
    else:
        if membership_obj.user in user_mapping:
            spec['user'] = user_mapping[membership_obj.user]
        else:
            raise ValueError(f"User ID {membership_obj.user} not specified in user mapping!")
    if dest_api is None:
        dest_api = src_api
    return dest_api.create_membership(dest_project, membership_spec=spec)



from uuid import uuid4
import pytest

import tator


def delete_attribute_helper(tator_api, type_getter, type_id, dtype):
    new_attr_name = f"New {dtype} {uuid4()}"
    entity_type = type_getter(type_id)

    addition = {
        "entity_type": f"{type(entity_type).__name__}",
        "addition": {"name": new_attr_name, "dtype": dtype},
    }
    if dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    tator_api.create_attribute_type(id=type_id, attribute_type_spec=addition)
    entity_type = type_getter(type_id)

    # Check for added attribute
    assert any(attr.name == new_attr_name for attr in entity_type.attribute_types)

    # Delete attribute
    deletion = {
        "entity_type": f"{type(entity_type).__name__}",
        "name": new_attr_name,
    }
    tator_api.delete_attribute_type(id=type_id, attribute_type_delete=deletion)
    entity_type = type_getter(type_id)

    # Check that the attribute is gone
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)


def delete_invalid_attribute_helper(tator_api, type_getter, type_id):
    new_attr_name = f"New {uuid4()}"
    entity_type = type_getter(type_id)

    # Check that the attribute does not exist
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)

    # Delete attribute
    deletion = {
        "entity_type": f"{type(entity_type).__name__}",
        "name": new_attr_name,
    }
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
        tator_api.delete_attribute_type(id=type_id, attribute_type_delete=deletion)

    # Check the exeption message for expected content
    assert "Could not find attribute name" in str(excinfo.value)


def test_box_type_delete_invalid_attribute(host, token, project, attribute_box_type):
    tator_api = tator.get_api(host, token)
    delete_invalid_attribute_helper(tator_api, tator_api.get_localization_type, attribute_box_type)


def test_state_type_delete_invalid_attribute(host, token, project, state_type):
    tator_api = tator.get_api(host, token)
    delete_invalid_attribute_helper(tator_api, tator_api.get_state_type, state_type)


def test_video_type_delete_invalid_attribute(host, token, project, attribute_video_type):
    tator_api = tator.get_api(host, token)
    delete_invalid_attribute_helper(tator_api, tator_api.get_media_type, attribute_video_type)


dtypes = ["int", "bool", "float", "string", "enum", "datetime", "geopos"]


@pytest.mark.parametrize("dtype", dtypes)
def test_box_type_delete_attribute(host, token, project, attribute_box_type, dtype):
    tator_api = tator.get_api(host, token)
    delete_attribute_helper(tator_api, tator_api.get_localization_type, attribute_box_type, dtype)



#!/usr/bin/env python

import argparse
import os
import shutil
import sys
from uuid import uuid1
import logging
import json
import glob
import re
import subprocess

from progressbar import progressbar
import requests
from tator.openapi.tator_openapi.models import MessageResponse

from ..util import md5sum
from ..util.get_api import get_api

from .create_media import create_media
from .determine_transcode import determine_transcode, update_media
from .transcode import convert_streaming
from .transcode import convert_archival
from .transcode import convert_audio
from .delete_media import delete_media
from .make_thumbnails import make_thumbnail_image, make_thumbnail_gif

if os.getenv("DD_LOGS_INJECTION"):
    import ddtrace.auto
    FORMAT = ('%(asctime)s %(levelname)s [%(name)s] [%(filename)s:%(lineno)d] '
              '[dd.service=%(dd.service)s dd.env=%(dd.env)s dd.version=%(dd.version)s dd.trace_id=%(dd.trace_id)s dd.span_id=%(dd.span_id)s] '
              '- %(message)s')
else:
    FORMAT = ('%(asctime)s %(levelname)s [%(name)s] [%(filename)s:%(lineno)d] '
              '- %(message)s')
logging.basicConfig(format=FORMAT)
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

def download_file(url, output_path):
    """
    Download a file using wget if available, otherwise use requests.

    Args:
        url: URL to download from
        output_path: Local path where file should be saved
    """
    # Try wget if available
    if shutil.which('wget'):
        try:
            wget_cmd = [
                'wget',
                '-c',                       # Continue/resume partial downloads
                '--tries=10',               # Retry up to 10 times
                '--retry-connrefused',      # Retry even if connection refused
                '--timeout=60',             # DNS/connect timeout (1 minute)
                '--read-timeout=600',       # Read timeout only if no data received (10 minutes)
                '-nv',                      # Quiet mode (suppress most wget output)
                '-O', output_path,          # Output file path
                url
            ]
            subprocess.run(wget_cmd, check=True, capture_output=True)
            logger.info(f"Successfully downloaded with wget: {output_path}")
            return
        except subprocess.CalledProcessError as e:
            logger.warning(f"wget download failed: {e}, falling back to requests")
            pass  # Fall back to requests

    # Fall back to requests
    response = requests.get(url, stream=True)
    response.raise_for_status()
    with open(output_path, "wb") as fp:
        for chunk in response.iter_content(chunk_size=10485760):  # 10 MiB
            if chunk:
                fp.write(chunk)

def parse_args():
    parser = argparse.ArgumentParser(description='Full transcode pipeline on a directory of files.')
    parser.add_argument('path', nargs='?', type=str,
                        help='Path to directory containing video files, or a video file, or text file with list of video files to be concatenated. '
                             'Ignored if --url is given.')
    parser.add_argument('--extension', type=str, default='mp4', help='File extension to upload. '
                                                                     'Ignored if path is a file.')
    parser.add_argument('--work_dir', type=str, help='Path to working directory. If not given, '
                                                     'the path containing videos will be used.')
    parser.add_argument('--url', type=str, help='URL where original file is hosted.')
    parser.add_argument('--host', type=str, default='https://cloud.tator.io', help='Host URL.')
    parser.add_argument('--token', type=str, help='REST API token.')
    parser.add_argument('--project', type=int, help='Unique integer specifying project ID.')
    parser.add_argument('--type', type=int, help='Unique integer specifying a media type.')
    parser.add_argument('--name', type=str, help='Name of the media.')
    parser.add_argument('--section', type=str, help='Media section name.')
    parser.add_argument('--section_id', type=int, help='Media section ID. If given `--section` is ignored.')
    parser.add_argument('--attributes', type=str, help="Attributes for media as a JSON string.")
    parser.add_argument('--media_id', type=int, help="Existing media ID, if applicable",
                        default=-1)
    parser.add_argument('--gid', type=str, help="Upload group ID.")
    parser.add_argument('--uid', type=str, help="Upload unique ID.")
    parser.add_argument('--group_to', type=int, default=1080,
                         help='Vertical resolutions below this will be transcoded with '
                              'multi-headed ffmpeg.')
    parser.add_argument('--email_spec', type=str, help="Optional email spec as a JSON string. The email will be sent upon completion of the transcode (see Email rest endpoint docs).")
    parser.add_argument('--cleanup', action='store_true',
                        help="Deletes working files after each file is transcoded and "
                             "uploaded.")
    parser.add_argument('--hwaccel', action='store_true', help="Use hardware acceleration.")
    parser.add_argument('--force_fps', type=float, default=-1, help='Force a specific fps for the video.')
    parser.add_argument(
        "--inhibit-upload", action="store_true", help="Do not upload the transcoded files to Tator."
    )
    parser.add_argument('--filter_complex', type=str, help='Allows (optionally templated) override of filter_complex argument to ffmpeg for streaming files. Substitutes {fps}, {width}, {height}, {hw_upload}, and {ridx} if those patterns are present, where width and height correspond to the output resolutions.')
    parser.add_argument('--bucket_id', type=int, help='Bucket ID if not default.')
    return parser.parse_args()

def get_file_paths(path, base):
    paths = {
        'original': path,
        'transcoded': base + '_transcoded',
        'thumbnail': base + '_thumbnail.jpg',
        'thumbnail_gif': base + '_thumbnail_gif.gif',
    }
    return paths

def compare_workloads(workloads_list):
    """
    Compare the parameters of workloads across multiple lists to ensure consistency.
    
    :param workloads_list: List of workload lists to compare.
    :return: True if all workloads are consistent, False otherwise.
    """
    if not workloads_list:
        return True  # No workloads to compare, considered consistent

    # Extract the first workload list as the reference
    reference_workloads = workloads_list[0]

    # Define a helper function to extract relevant parameters from a workload
    def extract_params(workload):
        return (
            workload['category'],
            workload['raw_height'],
            workload['raw_width'],
            tuple(workload['configs'])
        )

    # Extract parameters from the reference workloads
    reference_params = [extract_params(workload) for workload in reference_workloads]

    # Compare parameters with other workload lists
    for workloads in workloads_list[1:]:
        current_params = [extract_params(workload) for workload in workloads]
        if current_params != reference_params:
            return False  # Inconsistent parameters found

    return True  # All parameters are consistent

def transcode_single(path, args, gid):
    """Transcodes a single file.
    """
    logging.basicConfig(stream=sys.stdout, level=logging.INFO)

    # If a URL is given and path doesn't exist, download the file to path.
    if args.url:
        path = os.path.join(args.work_dir, args.name)
        logger.info(f"Downloading file from {args.url} to {path}")
        download_file(args.url, path)
    elif path is None:
        raise ValueError(f"Must provide one of --url or path!")

    fnames = None
    if isinstance(path, list):
        if args.name is None:
            raise ValueError(f"Must provide --name when path is a list!")
        if args.work_dir:
            base = os.path.join(args.work_dir, args.name)
        else:
            base = args.name
        paths = get_file_paths(path[0], base)

    elif os.path.splitext(path)[-1] == '.json':
        # This transcode will concat multiple files
        with open(path,'r') as fp:
            fnames = json.load(fp)
        assert args.name is not None, "args.name must be provided"

        # Get file paths.
        if args.work_dir:
            base = os.path.join(args.work_dir, os.path.splitext(os.path.basename(args.name))[0])
        else:
            base, _ = os.path.splitext(args.name)

        # Use the first input file
        paths = get_file_paths(fnames[0], base)

    else:
        # Get file paths.
        if args.work_dir:
            base = os.path.join(args.work_dir, os.path.splitext(os.path.basename(path))[0])
        else:
            base, _ = os.path.splitext(path)
        paths = get_file_paths(path, base)

        # Get md5 for the file.
    md5 = md5sum(paths['original'])

    # Get base filename.
    if args.name:
        name = args.name
    else:
        name = os.path.basename(paths['original'])

    # Create a uid if not set.
    if args.uid is None:
        uid = str(uuid1())
    else:
        uid = args.uid

    # Create the media object.
    if args.media_id == -1:
        kwargs = {}
        if args.section_id:
            kwargs['section_id'] = args.section_id
            args.section = None
        media_id = create_media(args.host, args.token, args.project, args.type, args.section, name, md5, gid, uid, args.attributes, args.url, **kwargs)
    else:
        media_id = args.media_id

    try:
        # Make thumbnails.
        make_thumbnail_image(
            args.host,
            args.token,
            media_id,
            paths["original"],
            paths["thumbnail"],
            inhibit_upload=args.inhibit_upload,
            bucket_id=args.bucket_id,
        )

        if fnames:
            workloads_list = []
            for fname in fnames:
                # Determine transcodes that need to be done.
                workloads = determine_transcode(
                    args.host, args.token, args.type, media_id, fname, group_to=args.group_to
                )
                workloads_list.append(workloads)
                if compare_workloads(workloads_list):
                    # Use the first workload, but update path
                    workloads = workloads_list[0]
                    for workload in workloads:
                        workload['path'] = fnames
        else:
            # Determine transcodes that need to be done.
            workloads = determine_transcode(
                args.host, args.token, args.type, media_id, path, group_to=args.group_to
            )

        # Transcode the video file.
        for workload in workloads:
            category = workload['category']
            del workload['category']
            del workload['id']
            if category == 'streaming':
                convert_streaming(
                    **workload,
                    host=args.host,
                    token=args.token,
                    media=media_id,
                    outpath=paths["transcoded"],
                    hwaccel=args.hwaccel,
                    force_fps=args.force_fps,
                    inhibit_upload=args.inhibit_upload,
                    filter_complex=getattr(args, "filter_complex", None),
                    bucket_id=args.bucket_id,
                )
            elif category == 'archival':
                del workload['configs']
                convert_archival(
                    **workload,
                    host=args.host,
                    token=args.token,
                    media=media_id,
                    outpath=paths["transcoded"],
                    hwaccel=args.hwaccel,
                    inhibit_upload=args.inhibit_upload,
                    bucket_id=args.bucket_id,
                )
            elif category == 'audio':
                del workload['configs']
                del workload['raw_width']
                del workload['raw_height']
                convert_audio(
                    **workload,
                    host=args.host,
                    token=args.token,
                    media=media_id,
                    outpath=paths["transcoded"],
                    inhibit_upload=args.inhibit_upload,
                    bucket_id=args.bucket_id,
                )

        # Get lowest resolution output for making the gif
        pattern = re.compile(r'^\d+\.mp4$')
        outputs = [os.path.basename(f) for f in glob.glob(os.path.join(paths['transcoded'], '*.mp4'))]
        outputs = list(filter(pattern.match, outputs))
        resolutions = [int(os.path.splitext(f)[0]) for f in outputs]
        if any([res > 256 for res in resolutions]):
            resolutions = [res for res in resolutions if res > 256]

        if len(resolutions) == 0:
            logger.info(f"No transcodes of streaming files was performed, skipping gif creation and media update.")
            return

        # Files are resolution height names, sort by lowest
        input_res = min(resolutions)

        # Make gif thumbnail
        make_thumbnail_gif(
            args.host,
            args.token,
            media_id,
            os.path.join(paths["transcoded"], f"{input_res}.mp4"),
            paths["thumbnail_gif"],
            inhibit_upload=args.inhibit_upload,
            bucket_id=args.bucket_id,
        )

        # Patch the media with the concatenated file
        max_res = max(resolutions)
        update_media(args.host, args.token, args.type, media_id, os.path.join(paths['transcoded'], f"{max_res}.mp4"))

    except Exception as exc:
        logging.error("Encountered exception!", exc_info=True)
        if args.media_id == -1:
            delete_media(args.host, args.token, media_id)
        raise RuntimeError(f"Transcode of file {path} failed!") from exc

    # Clean up after the transcode is finished (if enabled).
    if args.cleanup:
        for key in ['transcoded', 'thumbnail', 'thumbnail_gif']:
            path = paths[key]
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)

    # Send an email.
    if isinstance(args.email_spec, str):
        try:
            email_spec = json.loads(args.email_spec)
        except json.JSONDecodeError:
            logger.warning(
                "Could not decode email_spec string '%s'", args.email_spec, exc_info=True
            )
            email_spec = None
    else:
        email_spec = args.email_spec

    if email_spec:
        api = get_api(args.host, args.token)
        response = api.send_email(args.project, email_spec=email_spec)
        if isinstance(response, MessageResponse):
            logger.info(response.message)

def transcode_main(args):
    if args.gid is None:
        gid = str(uuid1())
    else:
        gid = args.gid
    if args.path and not isinstance(args.path, list) and os.path.isdir(args.path):
        file_list = []
        for root, _, files in os.walk(args.path):
            for fname in files:
                if fname.endswith(args.extension):
                    path = os.path.join(root, fname)
                    file_list.append(path)
        for path in progressbar(file_list):
            transcode_single(path, args, gid)
    else:
        transcode_single(args.path, args, gid)

if __name__ == '__main__':
    args = parse_args()
    transcode_main(args)



import logging
import os
import tempfile

import pytest
import tator
from tator.util._upload_file import _upload_file

logger = logging.getLogger(__name__)


def _upload_generic_file(
    host: str, token: str, project: int, file_id: int, file_path: str = "test.file"
):
    """Uploads the provided manifest file

    :param host: Project URL
    :param token: User token used for connecting to the host
    :param project: Unique identifier of test project
    :param file_id: Unique identifier of associated file
    :param file_path: Local file to be uploaded and saved
    :returns: Associated key of uploaded file
    """

    # Setup the interface to tator
    tator_api = tator.get_api(host=host, token=token)

    # Create the temporary file
    fd, local_file = tempfile.mkstemp()
    try:
        with os.fdopen(fd, "w") as file_handle:
            file_handle.write("Yo!")

        # Upload the file
        logger.info(f"Created temporary file: {local_file}")
        for progress, upload_info in _upload_file(
            api=tator_api, project=project, path=local_file, file_id=file_id
        ):
            logger.info(f"Upload progress: {progress}%")

    finally:
        os.remove(local_file)

    return upload_info.key


def test_file_type_delete(host: str, token: str, project: int) -> None:
    # Setup the interface to Tator
    tator_api = tator.get_api(host=host, token=token)

    # Test creating FileType objects with all available fields
    response = tator_api.create_file_type(
        project=project,
        file_type_spec={
            "name": "file_type_with_attrs",
            "description": "Test file type with attributes",
            "project": project,
            "attribute_types": [{"name": "test_bool", "dtype": "bool", "default": True}],
        },
    )
    file_type = response.id
    assert file_type >= 0

    # Verify no files exist to start
    response = tator_api.get_file_list(project, type=file_type)
    assert len(response) == 0

    file_ids = []
    for idx in range(10):
        file_spec = {
            "name": f"File_A_{idx}",
            "description": "hey",
            "type": file_type,
            "attributes": {"test_bool": True},
        }
        response = tator_api.create_file(project=project, file_spec=file_spec)
        key = _upload_generic_file(
            host=host, token=token, project=project, file_id=response.id, file_path="test.data"
        )
        tator_api.update_file(id=response.id, file_update={"path": key})
        file_ids.append({"file_id": response.id, "key": key})

    assert len(file_ids) == 10

    # Verify list is the right length
    response = tator_api.get_file_list(project, type=file_type)
    assert len(response) == 10

    response = tator_api.delete_file_type(file_type)

    assert str(file_type) in response.message, "File type id not found in delete response"
    assert "10" in response.message, "File count not found in delete response"

    try:
        caught_it = False
        response = tator_api.get_file_list(project, type=file_type)
    except:
        caught_it=True
    finally:
        assert caught_it



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Applet(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'categories': 'list[str]',
        'description': 'str',
        'html_file': 'str',
        'id': 'int',
        'name': 'str',
        'project': 'int',
        'rendered': 'str',
        'template': 'int',
        'tparams': 'list[Parameter]'
    }

    attribute_map = {
        'categories': 'categories',
        'description': 'description',
        'html_file': 'html_file',
        'id': 'id',
        'name': 'name',
        'project': 'project',
        'rendered': 'rendered',
        'template': 'template',
        'tparams': 'tparams'
    }

    def __init__(self, categories=None, description=None, html_file=None, id=None, name=None, project=None, rendered=None, template=None, tparams=[], local_vars_configuration=None):  # noqa: E501
        """Applet - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._categories = None
        self._description = None
        self._html_file = None
        self._id = None
        self._name = None
        self._project = None
        self._rendered = None
        self._template = None
        self._tparams = None
        self.discriminator = None

        if categories is not None:
            self.categories = categories
        if description is not None:
            self.description = description
        if html_file is not None:
            self.html_file = html_file
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if project is not None:
            self.project = project
        if rendered is not None:
            self.rendered = rendered
        self.template = template
        if tparams is not None:
            self.tparams = tparams

    @property
    def categories(self):
        """
        List of categories the applet belongs to

        :return: The categories of this Applet. 
        :rtype: list[str]
        """
        return self._categories

    @categories.setter
    def categories(self, categories):
        """
        List of categories the applet belongs to

        :param categories: The categories of this Applet.
        :type: list[str]
        """

        self._categories = categories

    @property
    def description(self):
        """
        Description of applet

        :return: The description of this Applet. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of applet

        :param description: The description of this Applet.
        :type: str
        """

        self._description = description

    @property
    def html_file(self):
        """
        Server URL to applet HTML file

        :return: The html_file of this Applet. 
        :rtype: str
        """
        return self._html_file

    @html_file.setter
    def html_file(self, html_file):
        """
        Server URL to applet HTML file

        :param html_file: The html_file of this Applet.
        :type: str
        """

        self._html_file = html_file

    @property
    def id(self):
        """
        Unique integer identifying the applet

        :return: The id of this Applet. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying the applet

        :param id: The id of this Applet.
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """
        Name of applet

        :return: The name of this Applet. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of applet

        :param name: The name of this Applet.
        :type: str
        """

        self._name = name

    @property
    def project(self):
        """
        Unique integer identifying the project associated with the applet

        :return: The project of this Applet. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying the project associated with the applet

        :param project: The project of this Applet.
        :type: int
        """

        self._project = project

    @property
    def rendered(self):
        """
        HTML format text containing rendered applet template. Only filled for single retrievals (not lists).

        :return: The rendered of this Applet. 
        :rtype: str
        """
        return self._rendered

    @rendered.setter
    def rendered(self, rendered):
        """
        HTML format text containing rendered applet template. Only filled for single retrievals (not lists).

        :param rendered: The rendered of this Applet.
        :type: str
        """

        self._rendered = rendered

    @property
    def template(self):
        """
        Unique integer identifying a hosted template. If set, `html_file` is ignored.

        :return: The template of this Applet. 
        :rtype: int
        """
        return self._template

    @template.setter
    def template(self, template):
        """
        Unique integer identifying a hosted template. If set, `html_file` is ignored.

        :param template: The template of this Applet.
        :type: int
        """

        self._template = template

    @property
    def tparams(self):
        """
        Template parameters used for rendering hosted template, if set.

        :return: The tparams of this Applet. 
        :rtype: list[Parameter]
        """
        return self._tparams

    @tparams.setter
    def tparams(self, tparams):
        """
        Template parameters used for rendering hosted template, if set.

        :param tparams: The tparams of this Applet.
        :type: list[Parameter]
        """

        self._tparams = tparams

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Applet):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Applet):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class ProjectUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'backup_bucket': 'int',
        'bucket': 'int',
        'elemental_id': 'str',
        'enable_downloads': 'bool',
        'name': 'str',
        'summary': 'str',
        'thumb': 'str',
        'upload_bucket': 'int'
    }

    attribute_map = {
        'backup_bucket': 'backup_bucket',
        'bucket': 'bucket',
        'elemental_id': 'elemental_id',
        'enable_downloads': 'enable_downloads',
        'name': 'name',
        'summary': 'summary',
        'thumb': 'thumb',
        'upload_bucket': 'upload_bucket'
    }

    def __init__(self, backup_bucket=None, bucket=None, elemental_id=None, enable_downloads=None, name=None, summary=None, thumb=None, upload_bucket=None, local_vars_configuration=None):  # noqa: E501
        """ProjectUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._backup_bucket = None
        self._bucket = None
        self._elemental_id = None
        self._enable_downloads = None
        self._name = None
        self._summary = None
        self._thumb = None
        self._upload_bucket = None
        self.discriminator = None

        if backup_bucket is not None:
            self.backup_bucket = backup_bucket
        if bucket is not None:
            self.bucket = bucket
        self.elemental_id = elemental_id
        if enable_downloads is not None:
            self.enable_downloads = enable_downloads
        if name is not None:
            self.name = name
        if summary is not None:
            self.summary = summary
        if thumb is not None:
            self.thumb = thumb
        if upload_bucket is not None:
            self.upload_bucket = upload_bucket

    @property
    def backup_bucket(self):
        """
        Unique integer identifying a bucket for backups.

        :return: The backup_bucket of this ProjectUpdate. 
        :rtype: int
        """
        return self._backup_bucket

    @backup_bucket.setter
    def backup_bucket(self, backup_bucket):
        """
        Unique integer identifying a bucket for backups.

        :param backup_bucket: The backup_bucket of this ProjectUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                backup_bucket is not None and backup_bucket < 1):  # noqa: E501
            raise ValueError("Invalid value for `backup_bucket`, must be a value greater than or equal to `1`")  # noqa: E501

        self._backup_bucket = backup_bucket

    @property
    def bucket(self):
        """
        Unique integer identifying a bucket.

        :return: The bucket of this ProjectUpdate. 
        :rtype: int
        """
        return self._bucket

    @bucket.setter
    def bucket(self, bucket):
        """
        Unique integer identifying a bucket.

        :param bucket: The bucket of this ProjectUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                bucket is not None and bucket < 1):  # noqa: E501
            raise ValueError("Invalid value for `bucket`, must be a value greater than or equal to `1`")  # noqa: E501

        self._bucket = bucket

    @property
    def elemental_id(self):
        """
        The elemental ID of the object.

        :return: The elemental_id of this ProjectUpdate. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        The elemental ID of the object.

        :param elemental_id: The elemental_id of this ProjectUpdate.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def enable_downloads(self):
        """
        Whether the UI should allow downloads for this project.

        :return: The enable_downloads of this ProjectUpdate. 
        :rtype: bool
        """
        return self._enable_downloads

    @enable_downloads.setter
    def enable_downloads(self, enable_downloads):
        """
        Whether the UI should allow downloads for this project.

        :param enable_downloads: The enable_downloads of this ProjectUpdate.
        :type: bool
        """

        self._enable_downloads = enable_downloads

    @property
    def name(self):
        """
        Name of the project.

        :return: The name of this ProjectUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the project.

        :param name: The name of this ProjectUpdate.
        :type: str
        """

        self._name = name

    @property
    def summary(self):
        """
        Summary of the project.

        :return: The summary of this ProjectUpdate. 
        :rtype: str
        """
        return self._summary

    @summary.setter
    def summary(self, summary):
        """
        Summary of the project.

        :param summary: The summary of this ProjectUpdate.
        :type: str
        """

        self._summary = summary

    @property
    def thumb(self):
        """
        S3 key of thumbnail used to represent the project.

        :return: The thumb of this ProjectUpdate. 
        :rtype: str
        """
        return self._thumb

    @thumb.setter
    def thumb(self, thumb):
        """
        S3 key of thumbnail used to represent the project.

        :param thumb: The thumb of this ProjectUpdate.
        :type: str
        """

        self._thumb = thumb

    @property
    def upload_bucket(self):
        """
        Unique integer identifying a bucket for uploads.

        :return: The upload_bucket of this ProjectUpdate. 
        :rtype: int
        """
        return self._upload_bucket

    @upload_bucket.setter
    def upload_bucket(self, upload_bucket):
        """
        Unique integer identifying a bucket for uploads.

        :param upload_bucket: The upload_bucket of this ProjectUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                upload_bucket is not None and upload_bucket < 1):  # noqa: E501
            raise ValueError("Invalid value for `upload_bucket`, must be a value greater than or equal to `1`")  # noqa: E501

        self._upload_bucket = upload_bucket

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProjectUpdate):
            return True

        return self.to_dict() != other.to_dict()



import os
import tempfile
import random
import uuid
import datetime

import tator

def random_localization(project, box_type, video_obj, post=False):
    x = random.uniform(0.0, 0.95)
    y = random.uniform(0.0, 0.95)
    w = random.uniform(0.05, 1.0 - x)
    h = random.uniform(0.05, 1.0 - y)
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        'test_float_array': [random.uniform(-1.0, 1.0) for _ in range(3)],
    }
    out = {
        'x': x,
        'y': y,
        'width': w,
        'height': h,
        'project': project,
        'type': box_type,
        'media_id': video_obj.id,
        'frame': random.randint(0, video_obj.num_frames - 1),
        'attributes': attributes
    }

    return {**out}

def test_state_graphic(host, token, project, video, box_type, track_type):
    api = tator.get_api(host, token)
    video_obj = api.get_media(video)
    localizations = [random_localization(project, box_type, video_obj, post=True) for _ in range(100)]
    response = api.create_localization_list(project, localizations)
    localization_ids = response.id
    track_spec = {
        'type': track_type,
        'localization_ids': localization_ids,
        'media_ids': [video],
    }
    tracks = api.create_state_list(project, body=track_spec)
    image = api.get_state_graphic(tracks.id[0])
    images = tator.util.full_state_graphic(api, tracks.id[0])
    api.delete_state_list(project, media_id=[video])
    api.delete_localization_list(project, media_id=[video])



import datetime
import random
import string
import uuid

import tator
from ._common import assert_close_enough


def random_leaf(project, leaf_type, parent_obj=None, post=False):
    attributes = {}
    name = "".join(random.choice(string.ascii_letters) for _ in range(10))
    out = {
        "project": project,
        "type": leaf_type,
        "name": name,
        "attributes": attributes
    }
    if parent_obj:
        out["parent"] = parent_obj.id

    return {**out}


def test_leaf_type_delete(host, token, project):
    tator_api = tator.get_api(host, token)

    response = tator_api.create_leaf_type(
        project,
        leaf_type_spec={
            "name": "leaf_type",
            "description": "Test leaf type",
            "attribute_types": [],
        },
    )
    leaf_type = response.id

    # Verify no leaves exist to start
    response = tator_api.get_leaf_list(project, type=leaf_type)
    assert len(response) == 0

    # Create root leaf.
    root_spec = random_leaf(project, leaf_type, None, True)
    response = tator_api.create_leaf_list(project=project, body=root_spec)
    assert isinstance(response, tator.models.CreateListResponse)
    prev_ids = response.id

    response = tator_api.delete_leaf_type(leaf_type)

    assert str(leaf_type) in response.message, "Leaf type id not found in delete response"
    assert "1" in response.message, "Leaf count not found in delete response"

    try:
        caught_it = False
        count = -1
        count = tator_api.get_leaf_count(project, type=leaf_type)
    except:
        caught_it=True
    finally:
        assert caught_it or count == 0



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class VideoClip(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'file': 'TemporaryFile',
        'segment_end_frames': 'list[int]',
        'segment_start_frames': 'list[int]'
    }

    attribute_map = {
        'file': 'file',
        'segment_end_frames': 'segment_end_frames',
        'segment_start_frames': 'segment_start_frames'
    }

    def __init__(self, file=None, segment_end_frames=None, segment_start_frames=None, local_vars_configuration=None):  # noqa: E501
        """VideoClip - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._file = None
        self._segment_end_frames = None
        self._segment_start_frames = None
        self.discriminator = None

        if file is not None:
            self.file = file
        if segment_end_frames is not None:
            self.segment_end_frames = segment_end_frames
        if segment_start_frames is not None:
            self.segment_start_frames = segment_start_frames

    @property
    def file(self):
        """

        :return: The file of this VideoClip. 
        :rtype: TemporaryFile
        """
        return self._file

    @file.setter
    def file(self, file):
        """

        :param file: The file of this VideoClip.
        :type: TemporaryFile
        """

        self._file = file

    @property
    def segment_end_frames(self):
        """
        List of end frames of segments that form the clip. Index associated with segment_start_frames.

        :return: The segment_end_frames of this VideoClip. 
        :rtype: list[int]
        """
        return self._segment_end_frames

    @segment_end_frames.setter
    def segment_end_frames(self, segment_end_frames):
        """
        List of end frames of segments that form the clip. Index associated with segment_start_frames.

        :param segment_end_frames: The segment_end_frames of this VideoClip.
        :type: list[int]
        """

        self._segment_end_frames = segment_end_frames

    @property
    def segment_start_frames(self):
        """
        List of start frames of segments that form the clip. Index associated with segment_end_frames.

        :return: The segment_start_frames of this VideoClip. 
        :rtype: list[int]
        """
        return self._segment_start_frames

    @segment_start_frames.setter
    def segment_start_frames(self, segment_start_frames):
        """
        List of start frames of segments that form the clip. Index associated with segment_end_frames.

        :param segment_start_frames: The segment_start_frames of this VideoClip.
        :type: list[int]
        """

        self._segment_start_frames = segment_start_frames

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, VideoClip):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, VideoClip):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Invitation(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_by': 'int',
        'created_datetime': 'datetime',
        'created_username': 'str',
        'email': 'str',
        'id': 'int',
        'organization': 'int',
        'permission': 'str',
        'registration_token': 'str',
        'status': 'str'
    }

    attribute_map = {
        'created_by': 'created_by',
        'created_datetime': 'created_datetime',
        'created_username': 'created_username',
        'email': 'email',
        'id': 'id',
        'organization': 'organization',
        'permission': 'permission',
        'registration_token': 'registration_token',
        'status': 'status'
    }

    def __init__(self, created_by=None, created_datetime=None, created_username=None, email=None, id=None, organization=None, permission=None, registration_token=None, status=None, local_vars_configuration=None):  # noqa: E501
        """Invitation - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._created_by = None
        self._created_datetime = None
        self._created_username = None
        self._email = None
        self._id = None
        self._organization = None
        self._permission = None
        self._registration_token = None
        self._status = None
        self.discriminator = None

        if created_by is not None:
            self.created_by = created_by
        if created_datetime is not None:
            self.created_datetime = created_datetime
        if created_username is not None:
            self.created_username = created_username
        if email is not None:
            self.email = email
        if id is not None:
            self.id = id
        if organization is not None:
            self.organization = organization
        if permission is not None:
            self.permission = permission
        if registration_token is not None:
            self.registration_token = registration_token
        if status is not None:
            self.status = status

    @property
    def created_by(self):
        """
        Unique integer identifying a user.

        :return: The created_by of this Invitation. 
        :rtype: int
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """
        Unique integer identifying a user.

        :param created_by: The created_by of this Invitation.
        :type: int
        """

        self._created_by = created_by

    @property
    def created_datetime(self):
        """
        Datetime this invitation was created.

        :return: The created_datetime of this Invitation. 
        :rtype: datetime
        """
        return self._created_datetime

    @created_datetime.setter
    def created_datetime(self, created_datetime):
        """
        Datetime this invitation was created.

        :param created_datetime: The created_datetime of this Invitation.
        :type: datetime
        """

        self._created_datetime = created_datetime

    @property
    def created_username(self):
        """
        Username of creator of the invitation.

        :return: The created_username of this Invitation. 
        :rtype: str
        """
        return self._created_username

    @created_username.setter
    def created_username(self, created_username):
        """
        Username of creator of the invitation.

        :param created_username: The created_username of this Invitation.
        :type: str
        """

        self._created_username = created_username

    @property
    def email(self):
        """
        Email address of invitee.

        :return: The email of this Invitation. 
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """
        Email address of invitee.

        :param email: The email of this Invitation.
        :type: str
        """

        self._email = email

    @property
    def id(self):
        """
        Unique integer identifying an invitation.

        :return: The id of this Invitation. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying an invitation.

        :param id: The id of this Invitation.
        :type: int
        """

        self._id = id

    @property
    def organization(self):
        """
        Unique integer identifying an organization.

        :return: The organization of this Invitation. 
        :rtype: int
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """
        Unique integer identifying an organization.

        :param organization: The organization of this Invitation.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                organization is not None and organization < 1):  # noqa: E501
            raise ValueError("Invalid value for `organization`, must be a value greater than or equal to `1`")  # noqa: E501

        self._organization = organization

    @property
    def permission(self):
        """
        User permission level for the organization.

        :return: The permission of this Invitation. 
        :rtype: str
        """
        return self._permission

    @permission.setter
    def permission(self, permission):
        """
        User permission level for the organization.

        :param permission: The permission of this Invitation.
        :type: str
        """
        allowed_values = ["Member", "Admin"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and permission not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `permission` ({0}), must be one of {1}"  # noqa: E501
                .format(permission, allowed_values)
            )

        self._permission = permission

    @property
    def registration_token(self):
        """
        Registration token.

        :return: The registration_token of this Invitation. 
        :rtype: str
        """
        return self._registration_token

    @registration_token.setter
    def registration_token(self, registration_token):
        """
        Registration token.

        :param registration_token: The registration_token of this Invitation.
        :type: str
        """

        self._registration_token = registration_token

    @property
    def status(self):
        """
        Status of the invitation.

        :return: The status of this Invitation. 
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """
        Status of the invitation.

        :param status: The status of this Invitation.
        :type: str
        """
        allowed_values = ["Pending", "Expired", "Accepted"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and status not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = status

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Invitation):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Invitation):
            return True

        return self.to_dict() != other.to_dict()



import tator
import time
from uuid import uuid1

def test_transcode_fault_handling(host, token, project):
    """ Verify Error Handling occurs for transcode endpoint """
    tator_api = tator.get_api(host, token)
    exception = False
    try:
        video_obj = tator_api.transcode(project,
                                        transcode_spec={'gid':str(uuid1()),
                                                        'md5':"1234",
                                                        'name':"blah",
                                                        'section':"blah",
                                                        'type':200,
                                                        'uid':str(uuid1()),
                                                        'url':"blah"})
    except Exception as e:
        exception = True
        code = e.status

    assert(exception)
    assert code >= 400

def test_transcode_existing_media(host, token, project, video_type, video_file):
    tator_api = tator.get_api(host, token)

    # Compute parameters needed to create media.
    md5 = tator.util.md5sum(video_file)
    upload_uid = str(uuid1())
    upload_gid = str(uuid1())
    fname='ExistingMediaTranscodeTest.mp4'
    section="Transcoded with Existing Media"

    # Define media spec.
    media_spec = {
        "type": video_type,
        "uid": upload_uid,
        "gid": upload_gid,
        "name": fname,
        "section": section,
        "md5": md5,
    }


    # Create the media.
    response = tator_api.create_media_list(project=project, body=media_spec)
    print(f"Transcoding video with existing media ID {response.id}...")
    for progress, response in tator.util.upload_media(
            tator_api, video_type, video_file, media_id=response.id
    ):
        print(f"Upload video progress: {progress}%")
    print(response.message)
    print(response.id)
    while True:
        transcode = tator_api.get_transcode(response.id)
        status = transcode.job.status.lower()
        if status == 'succeeded':
            break
        elif status == 'failed':
            raise Exception('Transcode of existing media failed!')
        else:
            print("Waiting for transcode of existing media to complete...")
            time.sleep(2.5)

def test_transcode_cancel(host, token, project, video_type, video_file):
    tator_api = tator.get_api(host, token)

    # Compute parameters needed to create media.
    md5 = tator.util.md5sum(video_file)
    upload_uid = str(uuid1())
    upload_gid = str(uuid1())
    fname='CancelTranscodeTest.mp4'
    section="Transcode cancel"

    # Define media spec.
    media_spec = {
        'type': video_type,
        'uid': upload_uid,
        'gid': upload_gid,
        'name': fname,
        'section': section,
        'md5': md5,
    }


    # Upload the same media multiple times.
    media_ids = []
    uids = []
    other_gid = str(uuid1())
    for idx in range(5):
        media_spec['uid'] = str(uuid1())
        if idx > 2:
            # On last two transcodes, change the group ID
            media_spec['gid'] = other_gid
        response = tator_api.create_media_list(project=project, body=[media_spec])
        media_ids.append(response.id[0])
        print(f"Transcoding video with existing media ID {response.id[0]}...")
        for progress, response in tator.util.upload_media(
            tator_api,
            video_type,
            video_file,
            upload_uid=media_spec['uid'],
            upload_gid=media_spec['gid'],
            media_id=response.id[0],
        ):
            print(f"Upload video progress: {progress}%")
        uids.append(response.id)

    # Get all transcodes for the two groups
    transcodes1 = tator_api.get_transcode_list(project, gid=upload_gid)
    transcodes2 = tator_api.get_transcode_list(project, gid=other_gid)
    assert(len(transcodes1) + len(transcodes2) == 5)

    # Cancel transcodes for one group
    before_delete = tator_api.get_transcode_list(project)
    response = tator_api.delete_transcode_list(project, gid=other_gid)
    print(response.message)
    remaining = tator_api.get_transcode_list(project)
    for transcode in remaining:
        assert(transcode.spec.gid != other_gid)
    assert(len(before_delete) - len(remaining)  >= 2)

    # Cancel individual transcode
    transcodes = tator_api.get_transcode_list(project, gid=upload_gid)
    uid = transcodes[0].spec.uid
    response = tator_api.delete_transcode(uid)
    print(response.message)
    remaining = tator_api.get_transcode_list(project, gid=upload_gid)
    assert(len(transcodes) - len(remaining) == 1)

    # Get transcodes by media
    transcodes = tator_api.get_transcode_list(project, media_id=media_ids)
    transcode = transcodes[0]
    media_id = transcode.spec.media_id
    uid = transcode.spec.uid
    transcode = tator_api.get_transcode(uid)
    assert(transcode.spec.media_id == media_id)
    response = tator_api.delete_transcode_list(project, media_id=[media_id])
    print(response.message)
    remaining = tator_api.get_transcode_list(project, media_id=media_ids)
    assert(len(transcodes) > len(remaining))
    response = tator_api.delete_transcode_list(project, media_id=media_ids)
    print(response.message)
    remaining = tator_api.get_transcode_list(project, media_id=media_ids)
    assert(len(remaining) == 0)



from pprint import pformat
import datetime
import random
from time import sleep
import uuid
from collections import Counter
import pytest

import tator
from ._common import assert_close_enough


def random_localization(project, box_type, video_obj, post=False):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        "test_bool": random.choice([False, True]),
        "test_int": random.randint(-1000, 1000),
        "test_float": random.uniform(-1000.0, 1000.0),
        "test_enum": random.choice(["a", "b", "c"]),
        "test_string": str(uuid.uuid1()),
        "test_datetime": datetime.datetime.now().isoformat(),
        "test_geopos": [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        "test_float_array": [random.uniform(-1.0, 1.0) for _ in range(3)],
    }
    out = {
        "x": x,
        "y": y,
        "width": w,
        "height": h,
        "project": project,
        "type": box_type,
        "media_id": video_obj.id,
        "frame": random.randint(0, video_obj.num_frames - 1),
        "attributes": attributes,
    }

    return {**out}


def comparison_query(tator_api, project, box_ids, exclude):
    """Runs a random query and compares results with ES enabled and disabled."""
    bool_value = random.choice([True, False])
    int_lower = random.randint(-1000, 0)
    int_upper = random.randint(0, 1000)
    float_lower = random.uniform(-1000.0, 0.0)
    float_upper = random.uniform(0.0, 1000.0)
    enum_value = random.choice(["a", "b", "c"])
    localization_id_query = {"ids": box_ids}
    attribute_filter = [
        f"test_bool::{'true' if bool_value else 'false'}",
        f"test_enum::{enum_value}",
    ]
    attribute_lte_filter = [f"test_int::{int_upper}"]
    attribute_gte_filter = [f"test_int::{int_lower}"]
    attribute_lt_filter = [f"test_float::{float_upper}"]
    attribute_gt_filter = [f"test_float::{float_lower}"]
    print("Starting PSQL query...")
    t0 = datetime.datetime.now()
    from_psql = tator_api.get_localization_list_by_id(
        project,
        localization_id_query=localization_id_query,
        attribute=attribute_filter,
        attribute_lte=attribute_lte_filter,
        attribute_gte=attribute_gte_filter,
        attribute_lt=attribute_lt_filter,
        attribute_gt=attribute_gt_filter,
    )
    psql_time = datetime.datetime.now() - t0
    print("Starting ES query...")
    t0 = datetime.datetime.now()
    from_es = tator_api.get_localization_list_by_id(
        project,
        localization_id_query=localization_id_query,
        attribute=attribute_filter,
        attribute_lte=attribute_lte_filter,
        attribute_gte=attribute_gte_filter,
        attribute_lt=attribute_lt_filter,
        attribute_gt=attribute_gt_filter,
    )
    es_time = datetime.datetime.now() - t0

    print("Checking PSQL and ES ids...")
    psql_ids = [ele.id for ele in from_psql]
    es_ids = [ele.id for ele in from_es]
    assert Counter(psql_ids) == Counter(es_ids)

    print("Checking PSQL and ES values...")
    assert len(from_psql) == len(from_es)
    for psql, es in zip(from_psql, from_es):
        assert_close_enough(psql, es, exclude)
        assert psql.attributes["test_bool"] == bool_value
        assert psql.attributes["test_int"] <= int_upper
        assert psql.attributes["test_int"] >= int_lower
        assert psql.attributes["test_float"] < float_upper
        assert psql.attributes["test_float"] > float_lower
        assert psql.attributes["test_enum"] == enum_value
    return psql_time, es_time


def test_localization_crud(host, token, project, video_type, video_temp, box_type):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video_temp)

    # These fields will not be checked for object equivalence after patch.
    exclude = ["project", "type", "media_id", "id", "type", "user", "ids", "in_place"]
    mapping = {"new_version": "version"}

    # Test bulk create.
    num_localization_randoms = [2424,8342,11231] # Pick a few random numbers to test bulk create.
    for num_localizations in num_localization_randoms:
        existing = len(tator_api.get_localization_list(project, type=box_type, media_id=[video_temp]))
        boxes = [
            random_localization(project, box_type, video_obj, post=True)
            for _ in range(num_localizations)
        ]
        box_ids = []
        for response in tator.util.chunked_create(
            tator_api.create_localization_list, project, body=boxes
        ):
            box_ids += response.id
        assert len(box_ids) == len(boxes)
        print(f"Created {len(box_ids)} boxes!")

        # Verify list is the right length
        response = tator_api.get_localization_list(project, type=box_type, media_id=[video_temp])
        assert len(response) == num_localizations + existing

        # Test media retrieval by localization ID.
        response = tator_api.get_media_list_by_id(project, {"localization_ids": box_ids})
        assert len(response) == 1
        assert response[0].id == video_temp

        # Test box retrieval by media ID.
        response = tator_api.get_localization_list_by_id(project, {"media_ids": [video_temp]})
        assert len(response) == len(box_ids)

        # Test single create.
        box = random_localization(project, box_type, video_obj, post=True)
        response = tator_api.create_localization_list(project, body=box)
        assert isinstance(response, tator.models.CreateListResponse)
        box_id = response.id[0]
        response = tator_api.get_localization(box_id)
        version_id = response.version
        elemental_id = response.elemental_id

        # Patch single box (in-place)
        patch = random_localization(project, box_type, video_obj)
        update_msg = {**patch, "in_place": 1}
        response = tator_api.update_localization(box_id, localization_update=update_msg)
        assert "message" in response.to_dict()
        print(response.message)

        # Get single box.
        updated_box = tator_api.get_localization(box_id)
        assert_close_enough(patch, updated_box, exclude)

        # Get box by ID.
        box_by_id = tator_api.get_localization_list_by_id(project, {"ids": [box_id]})
        assert len(box_by_id) == 1
        box_by_id = box_by_id[0]
        assert_close_enough(updated_box, box_by_id, exclude)

        # Patch single box via eid
        patch = random_localization(project, box_type, video_obj)
        response = tator_api.update_localization_by_elemental_id(
            version_id, elemental_id, localization_update=patch
        )
        assert "message" in response.to_dict()
        print(response.message)

        # Get single box.
        updated_box = tator_api.get_localization_by_elemental_id(version_id, elemental_id)
        assert_close_enough(patch, updated_box, exclude)

        # Delete single box.
        response = tator_api.delete_localization_by_elemental_id(version_id, elemental_id)
        assert "message" in response.to_dict()
        print(response.message)

        params = {"media_id": [video_temp], "type": box_type}
        assert tator_api.get_localization_count(project, **params) == len(boxes) + existing

        # Bulk update box attributes.
        response = tator_api.create_version(
            project,
            version_spec={
                "name": "Test Version",
                "description": "A version for testing",
            },
        )
        new_version = response.id
        bulk_patch = random_localization(project, box_type, video_obj)
        bulk_patch = {"attributes": bulk_patch["attributes"], "new_version": new_version, "in_place": 1}
        count = tator_api.get_localization_count(project, **params)
        with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException):
            response = tator_api.update_localization_list(
                project, **params, localization_bulk_update=bulk_patch, count=count + 1)
        response = tator_api.update_localization_list(
            project, **params, localization_bulk_update=bulk_patch, count=count
        )
        assert isinstance(response, tator.models.MessageResponse)
        print(response.message)

        # Bulk update specified boxes by ID.
        boxes = tator_api.get_localization_list(project, type=box_type, media_id=[video_obj.id])
        id_bulk_patch = random_localization(project, box_type, video_obj)
        box_ids = [x.id for x in boxes]
        update_ids = random.choices(box_ids, k=100)
        id_bulk_patch = {
            "attributes": id_bulk_patch["attributes"],
            "ids": update_ids,
            "new_version": new_version,
            "in_place": 1,
        }
        response = tator_api.update_localization_list(
            project, **params, localization_bulk_update=id_bulk_patch
        )
        assert isinstance(response, tator.models.MessageResponse)
        print(response.message)

        # Verify all boxes have been updated.
        boxes = tator_api.get_localization_list(project, **params, version=[new_version])
        dataframe = tator.util.to_dataframe(boxes)
        assert len(boxes) == len(dataframe)
        for box in boxes:
            if box.id in update_ids:
                assert_close_enough(id_bulk_patch, box, exclude, mapping)
            else:
                assert_close_enough(bulk_patch, box, exclude, mapping)

        # Clone boxes to same media.
        version_mapping = {version.id: version.id for version in tator_api.get_version_list(project)}
        generator = tator.util.clone_localization_list(
            tator_api,
            {**params, "project": project},
            project,
            version_mapping,
            {video_temp: video_temp},
            {box_type: box_type},
            tator_api,
        )
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} localizations...")
        print(f"Finished creating {num_created} localizations!")
        assert tator_api.get_localization_count(project, **params) == 2 * (len(boxes) + existing)

        # Delete all boxes.
        count = tator_api.get_localization_count(project, **params)
        with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException):
            response = tator_api.delete_localization_list(project, **params, count=count + 1)
        response = tator_api.delete_localization_list(project, **params, count=count)
        assert isinstance(response, tator.models.MessageResponse)

        # Verify all boxes are gone.
        boxes = tator_api.get_localization_list(project, **params)
        assert boxes == []

        boxes = tator_api.get_localization_list(project, **params, show_deleted=1, show_all_marks=1)
        assert boxes != []

        response = tator_api.delete_localization_list(
            project,
            **params,
            show_deleted=1,
            merge=0,
            show_all_marks=1,
            localization_bulk_delete={"prune": 1},
        )
        assert isinstance(response, tator.models.MessageResponse)

        # Clean up test version
        tator_api.delete_version(new_version)



import tempfile
import os

import tator

def test_attachment(host, token, project, video):
    tator_api = tator.get_api(host, token)

    with tempfile.NamedTemporaryFile(mode='w',suffix=".txt") as temp:
        temp.write("foo")
        temp.flush()
        for progress, response in tator.util.upload_attachment(tator_api, video, temp.name):
            print(f"Attachment upload progress: {progress}%")
        assert isinstance(response, tator.models.MessageResponse)
        print(f"Message: {response.message}")

    with tempfile.TemporaryDirectory() as temp_dir:
        temp_fp = os.path.join(temp_dir, "foo.txt")
        media_obj = tator_api.get_media(video)
        for progress in tator.util.download_attachment(tator_api, media_obj, temp_fp):
            print(f"Attachment download progress: {progress}%")
        with open(temp_fp, 'r') as temp_file:
            contents = temp_file.read()
            assert contents == "foo"



import tator
import time

def test_import_image(host, token, project, image_type):
    api = tator.get_api(host, token)
    url = 'https://tator-ci.s3.us-east-1.amazonaws.com/trip-summary.png'
    response = tator.util.import_media(api, image_type, url)
    assert(isinstance(response, tator.models.CreateListResponse))


def wait_for_transcode(api, transcode_id):
    while True:
        transcode = api.get_transcode(uid=transcode_id)
        status = transcode.job.status.lower()
        if status == "succeeded":
            break
        elif status == "failed":
            raise Exception("Media import failed!")
        else:
            print("Waiting for transcode of imported media to complete...")
            time.sleep(2.5)

def test_import_video(host, token, project, video_type):
    api = tator.get_api(host, token)
    url = 'https://tator-ci.s3.amazonaws.com/AudioVideoSyncTest_BallastMedia.mp4'
    response = tator.util.import_media(api, video_type, url, _request_timeout=30)
    print(response.message)
    wait_for_transcode(api, response.id)

    # Attempt to import the same video to the same media id, no additional transcodes should occur
    media_id = response.object['spec']['media_id']
    initial_obj = api.get_media(media_id)
    response = tator.util.import_media(api, video_type, url, media_id=media_id, _request_timeout=30)
    print(response.message)
    wait_for_transcode(api, response.id)
    final_obj = api.get_media(media_id)

    media_fields = ["archival", "_audio", "_streaming", "_thumbnail", "_thumbnail_gif"]

    for field in media_fields:
        assert (
            len(getattr(initial_obj.media_files, field)) ==
            len(getattr(final_obj.media_files, field))
        )

def test_import_video_to_section_id(host, token, project, video_type):
    api = tator.get_api(host, token)
    url = 'https://tator-ci.s3.amazonaws.com/AudioVideoSyncTest_BallastMedia.mp4'
    section_id = api.create_section(project, {"name":"Video import to section ID"}).id
    response = tator.util.import_media(api, video_type, url, _request_timeout=30, section_id=section_id)
    print(response.message)
    wait_for_transcode(api, response.id)

    # Attempt to import the same video to the same media id, no additional transcodes should occur
    media_id = response.object['spec']['media_id']
    initial_obj = api.get_media(media_id)
    response = tator.util.import_media(api, video_type, url, media_id=media_id, _request_timeout=30)
    print(response.message)
    wait_for_transcode(api, response.id)
    final_obj = api.get_media(media_id)

    media_fields = ["archival", "_audio", "_streaming", "_thumbnail", "_thumbnail_gif"]

    for field in media_fields:
        assert (
            len(getattr(initial_obj.media_files, field)) ==
            len(getattr(final_obj.media_files, field))
        )



# coding: utf-8

"""
    Tator REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.6.5
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class MediaUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'archive_state': 'str',
        'attributes': 'dict(str, object)',
        'codec': 'str',
        'concat': 'list[ConcatDefinition]',
        'elemental_id': 'str',
        'fps': 'float',
        'height': 'int',
        'last_edit_end': 'datetime',
        'last_edit_start': 'datetime',
        'live': 'LiveUpdateDefinition',
        'master_section': 'int',
        'media_files': 'MediaFiles',
        'multi': 'MultiDefinition',
        'name': 'str',
        'null_attributes': 'list[str]',
        'num_frames': 'int',
        'primary_section': 'int',
        'reset_attributes': 'list[str]',
        'summary_level': 'int',
        'user_elemental_id': 'str',
        'width': 'int'
    }

    attribute_map = {
        'archive_state': 'archive_state',
        'attributes': 'attributes',
        'codec': 'codec',
        'concat': 'concat',
        'elemental_id': 'elemental_id',
        'fps': 'fps',
        'height': 'height',
        'last_edit_end': 'last_edit_end',
        'last_edit_start': 'last_edit_start',
        'live': 'live',
        'master_section': 'master_section',
        'media_files': 'media_files',
        'multi': 'multi',
        'name': 'name',
        'null_attributes': 'null_attributes',
        'num_frames': 'num_frames',
        'primary_section': 'primary_section',
        'reset_attributes': 'reset_attributes',
        'summary_level': 'summary_level',
        'user_elemental_id': 'user_elemental_id',
        'width': 'width'
    }

    def __init__(self, archive_state=None, attributes=None, codec=None, concat=None, elemental_id=None, fps=None, height=None, last_edit_end=None, last_edit_start=None, live=None, master_section=None, media_files=None, multi=None, name=None, null_attributes=None, num_frames=None, primary_section=None, reset_attributes=None, summary_level=None, user_elemental_id=None, width=None, local_vars_configuration=None):  # noqa: E501
        """MediaUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._archive_state = None
        self._attributes = None
        self._codec = None
        self._concat = None
        self._elemental_id = None
        self._fps = None
        self._height = None
        self._last_edit_end = None
        self._last_edit_start = None
        self._live = None
        self._master_section = None
        self._media_files = None
        self._multi = None
        self._name = None
        self._null_attributes = None
        self._num_frames = None
        self._primary_section = None
        self._reset_attributes = None
        self._summary_level = None
        self._user_elemental_id = None
        self._width = None
        self.discriminator = None

        if archive_state is not None:
            self.archive_state = archive_state
        if attributes is not None:
            self.attributes = attributes
        if codec is not None:
            self.codec = codec
        if concat is not None:
            self.concat = concat
        if elemental_id is not None:
            self.elemental_id = elemental_id
        if fps is not None:
            self.fps = fps
        if height is not None:
            self.height = height
        if last_edit_end is not None:
            self.last_edit_end = last_edit_end
        if last_edit_start is not None:
            self.last_edit_start = last_edit_start
        if live is not None:
            self.live = live
        if master_section is not None:
            self.master_section = master_section
        if media_files is not None:
            self.media_files = media_files
        if multi is not None:
            self.multi = multi
        if name is not None:
            self.name = name
        if null_attributes is not None:
            self.null_attributes = null_attributes
        if num_frames is not None:
            self.num_frames = num_frames
        if primary_section is not None:
            self.primary_section = primary_section
        if reset_attributes is not None:
            self.reset_attributes = reset_attributes
        if summary_level is not None:
            self.summary_level = summary_level
        if user_elemental_id is not None:
            self.user_elemental_id = user_elemental_id
        if width is not None:
            self.width = width

    @property
    def archive_state(self):
        """
        Marks media for archival or retrieval. Media may not be set directly to `live` or `archived`, the system performs that transition for the user.

        :return: The archive_state of this MediaUpdate. 
        :rtype: str
        """
        return self._archive_state

    @archive_state.setter
    def archive_state(self, archive_state):
        """
        Marks media for archival or retrieval. Media may not be set directly to `live` or `archived`, the system performs that transition for the user.

        :param archive_state: The archive_state of this MediaUpdate.
        :type: str
        """
        allowed_values = ["to_archive", "to_live"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and archive_state not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `archive_state` ({0}), must be one of {1}"  # noqa: E501
                .format(archive_state, allowed_values)
            )

        self._archive_state = archive_state

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this MediaUpdate. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this MediaUpdate.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def codec(self):
        """
        Codec of the original video.

        :return: The codec of this MediaUpdate. 
        :rtype: str
        """
        return self._codec

    @codec.setter
    def codec(self, codec):
        """
        Codec of the original video.

        :param codec: The codec of this MediaUpdate.
        :type: str
        """

        self._codec = codec

    @property
    def concat(self):
        """
        List of concated videos

        :return: The concat of this MediaUpdate. 
        :rtype: list[ConcatDefinition]
        """
        return self._concat

    @concat.setter
    def concat(self, concat):
        """
        List of concated videos

        :param concat: The concat of this MediaUpdate.
        :type: list[ConcatDefinition]
        """

        self._concat = concat

    @property
    def elemental_id(self):
        """
        Unique ID of an element

        :return: The elemental_id of this MediaUpdate. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        Unique ID of an element

        :param elemental_id: The elemental_id of this MediaUpdate.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def fps(self):
        """
        Frame rate of the video.

        :return: The fps of this MediaUpdate. 
        :rtype: float
        """
        return self._fps

    @fps.setter
    def fps(self, fps):
        """
        Frame rate of the video.

        :param fps: The fps of this MediaUpdate.
        :type: float
        """

        self._fps = fps

    @property
    def height(self):
        """
        Pixel height of the video.

        :return: The height of this MediaUpdate. 
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """
        Pixel height of the video.

        :param height: The height of this MediaUpdate.
        :type: int
        """

        self._height = height

    @property
    def last_edit_end(self):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :return: The last_edit_end of this MediaUpdate. 
        :rtype: datetime
        """
        return self._last_edit_end

    @last_edit_end.setter
    def last_edit_end(self, last_edit_end):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :param last_edit_end: The last_edit_end of this MediaUpdate.
        :type: datetime
        """

        self._last_edit_end = last_edit_end

    @property
    def last_edit_start(self):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :return: The last_edit_start of this MediaUpdate. 
        :rtype: datetime
        """
        return self._last_edit_start

    @last_edit_start.setter
    def last_edit_start(self, last_edit_start):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :param last_edit_start: The last_edit_start of this MediaUpdate.
        :type: datetime
        """

        self._last_edit_start = last_edit_start

    @property
    def live(self):
        """

        :return: The live of this MediaUpdate. 
        :rtype: LiveUpdateDefinition
        """
        return self._live

    @live.setter
    def live(self, live):
        """

        :param live: The live of this MediaUpdate.
        :type: LiveUpdateDefinition
        """

        self._live = live

    @property
    def master_section(self):
        """
        Unique ID to define the ACL origin of this object. If not set defaults to the master section of the primary_section.

        :return: The master_section of this MediaUpdate. 
        :rtype: int
        """
        return self._master_section

    @master_section.setter
    def master_section(self, master_section):
        """
        Unique ID to define the ACL origin of this object. If not set defaults to the master section of the primary_section.

        :param master_section: The master_section of this MediaUpdate.
        :type: int
        """

        self._master_section = master_section

    @property
    def media_files(self):
        """

        :return: The media_files of this MediaUpdate. 
        :rtype: MediaFiles
        """
        return self._media_files

    @media_files.setter
    def media_files(self, media_files):
        """

        :param media_files: The media_files of this MediaUpdate.
        :type: MediaFiles
        """

        self._media_files = media_files

    @property
    def multi(self):
        """

        :return: The multi of this MediaUpdate. 
        :rtype: MultiDefinition
        """
        return self._multi

    @multi.setter
    def multi(self, multi):
        """

        :param multi: The multi of this MediaUpdate.
        :type: MultiDefinition
        """

        self._multi = multi

    @property
    def name(self):
        """
        Name of the media.

        :return: The name of this MediaUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the media.

        :param name: The name of this MediaUpdate.
        :type: str
        """

        self._name = name

    @property
    def null_attributes(self):
        """
        Null a value in the attributes body

        :return: The null_attributes of this MediaUpdate. 
        :rtype: list[str]
        """
        return self._null_attributes

    @null_attributes.setter
    def null_attributes(self, null_attributes):
        """
        Null a value in the attributes body

        :param null_attributes: The null_attributes of this MediaUpdate.
        :type: list[str]
        """

        self._null_attributes = null_attributes

    @property
    def num_frames(self):
        """
        Number of frames in the video.

        :return: The num_frames of this MediaUpdate. 
        :rtype: int
        """
        return self._num_frames

    @num_frames.setter
    def num_frames(self, num_frames):
        """
        Number of frames in the video.

        :param num_frames: The num_frames of this MediaUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                num_frames is not None and num_frames < 0):  # noqa: E501
            raise ValueError("Invalid value for `num_frames`, must be a value greater than or equal to `0`")  # noqa: E501

        self._num_frames = num_frames

    @property
    def primary_section(self):
        """
        Unique ID for folder section containing this media, if any. When patching, use -1 to indicate no section.

        :return: The primary_section of this MediaUpdate. 
        :rtype: int
        """
        return self._primary_section

    @primary_section.setter
    def primary_section(self, primary_section):
        """
        Unique ID for folder section containing this media, if any. When patching, use -1 to indicate no section.

        :param primary_section: The primary_section of this MediaUpdate.
        :type: int
        """

        self._primary_section = primary_section

    @property
    def reset_attributes(self):
        """
        Reset an attribute to the default value specified in the Type object

        :return: The reset_attributes of this MediaUpdate. 
        :rtype: list[str]
        """
        return self._reset_attributes

    @reset_attributes.setter
    def reset_attributes(self, reset_attributes):
        """
        Reset an attribute to the default value specified in the Type object

        :param reset_attributes: The reset_attributes of this MediaUpdate.
        :type: list[str]
        """

        self._reset_attributes = reset_attributes

    @property
    def summary_level(self):
        """
        If supplied, this video is best summarized at this frame interval

        :return: The summary_level of this MediaUpdate. 
        :rtype: int
        """
        return self._summary_level

    @summary_level.setter
    def summary_level(self, summary_level):
        """
        If supplied, this video is best summarized at this frame interval

        :param summary_level: The summary_level of this MediaUpdate.
        :type: int
        """

        self._summary_level = summary_level

    @property
    def user_elemental_id(self):
        """
        Unique ID of the original user who created this. If permissions allow, will change the creating user to the one referenced by this elemental_id

        :return: The user_elemental_id of this MediaUpdate. 
        :rtype: str
        """
        return self._user_elemental_id

    @user_elemental_id.setter
    def user_elemental_id(self, user_elemental_id):
        """
        Unique ID of the original user who created this. If permissions allow, will change the creating user to the one referenced by this elemental_id

        :param user_elemental_id: The user_elemental_id of this MediaUpdate.
        :type: str
        """

        self._user_elemental_id = user_elemental_id

    @property
    def width(self):
        """
        Pixel width of the video.

        :return: The width of this MediaUpdate. 
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """
        Pixel width of the video.

        :param width: The width of this MediaUpdate.
        :type: int
        """

        self._width = width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MediaUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MediaUpdate):
            return True

        return self.to_dict() != other.to_dict()
